c  Program hardisp
c  
c  Reads in a file of station displacements in the BLQ format used by Scherneck
c   and Bos for ocean loading, and outputs a time series of computed
c   tidal displacments, using an expanded set of tidal constituents, whose
c   amplitudes and phases are found by spline interpolation of the tidal
c   admittance.  A total of 141 constituent tides are included, which gives
c   a precision of about 1%
c
c  See format statement 100, below, for a summary of usage
c
c  Duncan Agnew, August 2005, based on the program   hartid  distributed
c   with the SPOTL loading package.
c
c  Corrected Nov 2005 (error in reading displacements)
c  Revised   Dec 2005 (Hartmann-Wenzel harmonics in admint)
c     GP  17 Dec 2007 Corrected 'intial' to 'initial' (noted by T. Springer)
c
c  This software is freely provided for non-profit use only.  All commercial
c   uses and any other restrictions or limitations imposed by third parties
c   are forbidden.  No warranty is made or implied.
c
c    **calls admint,tdfrph,recurs,spline,shells
c
c  NOTE: The st & si arrays and nstep parameter, in subroutine etutc, must be
c   updated for new leap seconds; currently complete through 2006.0.
c
c  Validation test data using hardisp.f routine:
c   Test made by Jim Ray, 31 January 2006, using f90 compiler on HP-UX 11.11
c   and verified by several other people using a variety of platforms.
c
c   Input ocean loading coefficients were generated by the ocean loading
c   service on 30 January 2006 using http://www.oso.chalmers.se/~loading/
c   for IGS station BRST using the CSR4.0 model and "NO" geocenter correction:
c
c  BRST
c$$ CSR4.0_f_PP ID: Jan  30, 2006 15:51
c$$ Computed by OLMPP by H G Scherneck, Onsala Space Observatory, 2006
c$$ BRST,                                RADI TANG lon/lat:  355.5034   48.3805
c  .03979 .01305 .00822 .00338 .00440 .00225 .00144 .00057 .00066 .00034 .00030
c  .00876 .00289 .00186 .00075 .00068 .00051 .00023 .00015 .00006 .00002 .00003
c  .00633 .00263 .00137 .00071 .00041 .00033 .00014 .00010 .00004 .00002 .00001
c   -61.9  -25.3  -81.5  -21.9  -72.3  160.2  -78.0  105.6    9.1    6.1    1.6
c    97.2  131.1   76.9  133.7   91.2  -13.8   84.0  -67.1 -171.4 -165.9 -178.5
c    32.6   65.9   18.4   68.3   20.7  -91.1    8.5 -156.8  -15.8   -3.3   -0.8
c
c   The hardisp routine was executed with the following command and with the
c   six lines of coefficients above input from standard input.  In other words,
c   BRST site displacements were computed for epoch 2006 01 01 00h 00m 00s and
c   for each of the following 23 hours.  The output given below was written to
c   standard output in units of meters with the format 3F14.6:
c         dU    dS    dW
c
c> hardisp 2006 1 1 0 0 0 24 3600
c
c      0.042103      0.005660     -0.005394
c      0.018999      0.008682      0.000630
c     -0.009315      0.009647      0.006688
c     -0.035991      0.008280      0.011288
c     -0.054513      0.004885      0.013287
c     -0.060329      0.000276      0.012176
c     -0.051989     -0.004425      0.008213
c     -0.031523     -0.008072      0.002357
c     -0.003937     -0.009776     -0.003966
c      0.024034     -0.009133     -0.009219
c      0.045603     -0.006322     -0.012142
c      0.055629     -0.002061     -0.012060
c      0.051896      0.002574     -0.009054
c      0.035656      0.006422     -0.003938
c      0.011297      0.008527      0.001950
c     -0.014774      0.008380      0.007087
c     -0.035767      0.006049      0.010153
c     -0.046230      0.002156      0.010362
c     -0.043444     -0.002278      0.007665
c     -0.028111     -0.006094      0.002764
c     -0.004190     -0.008291     -0.003068
c      0.022105     -0.008278     -0.008309
c      0.043879     -0.006030     -0.011570
c      0.055307     -0.002094     -0.011957
c
c
c
      character*40 dumm
      double precision f,pz,ps,pw,dr,pi,scr
      dimension tamp(3,11),tph(3,11)
      dimension idt(6,11),amp(11),phase(11)
      dimension az(141),pz(141),hcz(282)
      dimension as(141),ps(141),hcs(282)
      dimension aw(141),pw(141),hcw(282)
      dimension f(141),dz(600),ds(600),dw(600),scr(423),wf(141)
      common/date/it(5)
      data nt/141/,dr/.01745329252d0/,pi/3.1415926535898d0/,irli/1/
      data ntin/11/
      data luo/6/
c  Cartwright-Tayler numbers of tides used in Scherneck lists:
c      M2, S2, N2, K2, K1, O1, P1, Q1, Mf, Mm, Ssa
      data idt/
     1  2, 0, 0, 0, 0, 0,   2, 2,-2, 0, 0, 0,   2,-1, 0, 1, 0, 0,  
     2  2, 2, 0, 0, 0, 0,   1, 1, 0, 0, 0, 0,   1,-1, 0, 0, 0, 0,
     3  1, 1,-2, 0, 0, 0,   1,-2, 0, 1, 0, 0,   0, 2, 0, 0, 0, 0,
     4  0, 1, 0,-1, 0, 0,   0, 0, 2, 0, 0, 0/
c*******************************************************************
c
c  check number of arguments from command line, then read them in
c
c*******************************************************************
      if(iargc().lt.7.or.iargc().gt.8) then
	 write(luo,100)
 100  format(/,'Usage:',/,
     1  '   hardisp yr [d-of-yr | month day] hr min sec num samp',//,
     1  ' Where ',/,
     2  '   the UTC date given is the time of the first term output',/,
     4  '   num is the number of output epochs to be written out',/,
     3  '   samp is the sample interval (seconds)',//,
     5  '  The harmonics file (amp and phase of displacement) is ',/,
     6  '    read from standard input in the BLQ format used by  ',/,
     6  '    Scherneck and Bos                                   ',//,
     8  '  Results are written to standard output (units = m):',/,
     9  '      dU    dS    dW   ',/,
     1  '    using format: 3F14.6 ',/)
	 stop
      endif
      call getarg(1,dumm)
      read(dumm,102) it(1)
 102  format(i4)
      if(iargc().eq.7) then
        call getarg(2,dumm)
        read(dumm,102) it(2)
        nb=0
      endif
      if(iargc().eq.8) then
        call getarg(2,dumm)
        read(dumm,102) imonth
        call getarg(3,dumm)
        read(dumm,102) iday
        nb=1
        it(2) = iday + mday(it(1),imonth)
      endif
      call getarg(nb+3,dumm)
      read(dumm,102) it(3)
      call getarg(nb+4,dumm)
      read(dumm,102) it(4)
      call getarg(nb+5,dumm)
      read(dumm,102) it(5)
      call getarg(nb+6,dumm)
      read(dumm,104) irnt
 104  format(i6)
      call getarg(nb+7,dumm)
      read(dumm,106) samp
 106  format(f7.0)
c*******************************************************************
c
c  read in amplitudes and phases, in standard "Scherneck" form, from
c    standard input
c
c*******************************************************************
      do i=1,3
        read(5,108) (tamp(i,kk),kk=1,11)
 108    format(1x,11f7.5)
      enddo
      do i=1,3
        read(5,110) (tph(i,kk),kk=1,11)
 110    format(1x,11f7.1)
c
c change sign for phase, to be negative for lags
c
        do kk=1,ntin
          tph(i,kk)=-tph(i,kk)
        enddo
      enddo
c*******************************************************************
c
c  find amplitudes and phases for all constituents, for each of the
c    three displacements. Note that the same frequencies are returned each
c    time
c  BLQ format order is vertical, horizontal EW, horizontal NS
c
c*******************************************************************
      do i=1,ntin
        amp(i)=tamp(1,i)
        phase(i)=tph(1,i)
      enddo
      call admint(amp,idt,phase,az,f,pz,ntin,ntout)
      do i=1,ntin
        amp(i)=tamp(2,i)
        phase(i)=tph(2,i)
      enddo
      call admint(amp,idt,phase,aw,f,pw,ntin,ntout)
      do i=1,ntin
        amp(i)=tamp(3,i)
        phase(i)=tph(3,i)
      enddo
      call admint(amp,idt,phase,as,f,ps,ntin,ntout)
c
c  set up for recursion, by normalizing frequencies, and converting
c  phases to radians
c
      do i=1,ntout
        pz(i) = dr*pz(i)
        ps(i) = dr*ps(i)
        pw(i) = dr*pw(i)
        f(i) = samp*pi*f(i)/43200.d0
        wf(i) = f(i)
      enddo
c*******************************************************************
c
c  Loop over times, 600 output points at a time. At the start of each
c    such block, convert from amp and phase to sin and cos (hc array) at
c    the start of the block. The computation of values within each
c    block is done recursively, since the times are equi-spaced.
c
c*******************************************************************

 31   irhi = min(irli+599,irnt)
      np = irhi - irli + 1
c set up harmonic coefficients, compute tide, and write out
      do i=1,nt
        hcz(2*i-1) = az(i)*dcos(pz(i))
        hcz(2*i)  = -az(i)*dsin(pz(i))
        hcs(2*i-1) = as(i)*dcos(ps(i))
        hcs(2*i)  = -as(i)*dsin(ps(i))
        hcw(2*i-1) = aw(i)*dcos(pw(i))
        hcw(2*i)  = -aw(i)*dsin(pw(i))
      enddo
      call recurs(dz,np,hcz,ntout,wf,scr)
      call recurs(ds,np,hcs,ntout,wf,scr)
      call recurs(dw,np,hcw,ntout,wf,scr)
      write(luo,120) (dz(i),ds(i),dw(i),i=1,np)
 120  format(3f14.6)
      if(irhi.eq.irnt) stop
      irli = irhi + 1
c  reset phases to the start of the new section
      do i=1,nt
        pz(i) = dmod(pz(i) + np*f(i),2.d0*pi)
        ps(i) = dmod(ps(i) + np*f(i),2.d0*pi)
        pw(i) = dmod(pw(i) + np*f(i),2.d0*pi)
      enddo
      go to 31
      end
      function mday(iy,m)
c
c  finds day number of day before start of month m, of year iy, in
c   Gregorian intercalation
c
      leap = 1 - (mod(iy,4)+3)/4
      if(mod(iy,100).eq.0.and.mod(iy,400).ne.0) leap=0
      mday = mod((367*(m-2-12*((m-14)/12)))/12+29,365) + leap*((9+m)/12)
      return
      end
      subroutine admint(ampin,idtin,phin,amp,f,p,nin,nout)
c  Returns the amplitude, frequency, and phase of a set of tidal
c  constituents. n is input as the number wanted, and returned as the number
c  provided.  The constituents used are stored in the arrays idd (doodson
c  number) and tamp (Cartwright-Edden amplitude).  The actual amp and
c  phase of each of these are determined by spline interpolation of the
c  real and imaginary part of the admittance, as specified at a subset
c  of the constituents.
c  The phase is determined for a time set in common block /date/ (see
c  subroutine tdfrph), outside of this subroutine.
c   Note that the arrays f and p must be specified as double precision.
c
c   $$$$calls tdfrph,spline,eval,shells
c
      double precision f,p,fr,pr
      dimension ampin(*),idtin(6,*),phin(*)
      dimension amp(*),f(*),p(*)
c  arrays containing information about all stored constituents
      dimension idd(6,141),tamp(141)
c  arrays containing information about the subset whose amp and phase may
c  be specified, and scratch arrays for the spline routines
c   at most 20 constituents may be specified.
      dimension rl(20),aim(20),rf(20),key(20),scr(20),
     1 zdi(20),zdr(20),di(20),dr(20),sdi(20),sdr(20),tdi(20),tdr(20)
      common/locat/rlong
      data dtr/.017453293/
      data nt/141/,ncon/20/
      data rl/20*0.0/,aim/20*0.0/,rf/20*0.0/
      data zdi/20*0.0/,zdr/20*0.0/,di/20*0.0/,dr/20*0.0/
      data sdi/20*0.0/,sdr/20*0.0/,tdi/20*0.0/,tdr/20*0.0/
      data tamp/
     1 0.63221, .29411, .12105, .07991, .02382,-.02359, .02299, .01933,
     1 -.01787, .01719, .01602, .00467,-.00466,-.00452, .00447, .00447,
     1 0.00259,-.00246,-.00217, .00197, .00195, .00191,-.00190, .00180,
     1 0.00130, .00117, .00113, .00106,-.00102,-.00102, .00101, .00090,
     1 -.00086, .00085, .00085, .00077, .00074, .00074,-.00072, .00070,
     1 0.00066, .00065,-.00065, .00063, .00063,-.00060, .00059, .00054,
     1 0.00048,-.00046, .00041, .36864,-.26223,-.12200,-.05021, .05003,
     1 -.04947, .02062, .02061, .01128,-.00953,-.00947,-.00801, .00741,
     1 -.00730, .00723,-.00713,-.00664, .00525, .00414, .00409, .00394,
     1 0.00394, .00342, .00342, .00289, .00288, .00216,-.00194, .00193,
     1 -.00180, .00169, .00169, .00152, .00151,-.00151, .00138, .00137,
     1 0.00137,-.00125,-.00107, .00102, .00090, .00087,-.00079, .00079,
     1 0.00078,-.00075,-.00075, .00067,-.00060,-.00060, .00054, .00054,
     1 -.00054,-.00047,-.00044, .00044, .00042, .00041,-.06661,-.03518,
     1 -.03099, .02793,-.02762,-.01275,-.00673,-.00584,-.00529,-.00492,
     1 -.00288,-.00258,-.00242, .00231, .00228,-.00204, .00188,-.00181,
     1 -.00169,-.00100,-.00093,-.00084, .00077, .00077,-.00070,-.00050,
     1 -.00049, .00049, .00048, .00044,-.00042/
      data idd/
     1  2, 0, 0, 0, 0, 0,   2, 2,-2, 0, 0, 0,   2,-1, 0, 1, 0, 0,  
     1  2, 2, 0, 0, 0, 0,   2, 2, 0, 0, 1, 0,   2, 0, 0, 0,-1, 0,  
     1  2,-1, 2,-1, 0, 0,   2,-2, 2, 0, 0, 0,   2, 1, 0,-1, 0, 0,  
     1  2, 2,-3, 0, 0, 1,   2,-2, 0, 2, 0, 0,   2,-3, 2, 1, 0, 0,  
     1  2, 1,-2, 1, 0, 0,   2,-1, 0, 1,-1, 0,   2, 3, 0,-1, 0, 0,  
     1  2, 1, 0, 1, 0, 0,   2, 2, 0, 0, 2, 0,   2, 2,-1, 0, 0,-1,  
     1  2, 0,-1, 0, 0, 1,   2, 1, 0, 1, 1, 0,   2, 3, 0,-1, 1, 0,  
     1  2, 0, 1, 0, 0,-1,   2, 0,-2, 2, 0, 0,   2,-3, 0, 3, 0, 0,  
     1  2,-2, 3, 0, 0,-1,   2, 4, 0, 0, 0, 0,   2,-1, 1, 1, 0,-1,  
     1  2,-1, 3,-1, 0,-1,   2, 2, 0, 0,-1, 0,   2,-1,-1, 1, 0, 1,  
     1  2, 4, 0, 0, 1, 0,   2,-3, 4,-1, 0, 0,   2,-1, 2,-1,-1, 0,  
     1  2, 3,-2, 1, 0, 0,   2, 1, 2,-1, 0, 0,   2,-4, 2, 2, 0, 0,  
     1  2, 4,-2, 0, 0, 0,   2, 0, 2, 0, 0, 0,   2,-2, 2, 0,-1, 0,  
     1  2, 2,-4, 0, 0, 2,   2, 2,-2, 0,-1, 0,   2, 1, 0,-1,-1, 0,  
     1  2,-1, 1, 0, 0, 0,   2, 2,-1, 0, 0, 1,   2, 2, 1, 0, 0,-1,  
     1  2,-2, 0, 2,-1, 0,   2,-2, 4,-2, 0, 0,   2, 2, 2, 0, 0, 0,  
     1  2,-4, 4, 0, 0, 0,   2,-1, 0,-1,-2, 0,   2, 1, 2,-1, 1, 0,  
     1  1, 1, 0, 0, 0, 0,   1,-1, 0, 0, 0, 0,   1, 1,-2, 0, 0, 0,  
     1  1,-2, 0, 1, 0, 0,   1, 1, 0, 0, 1, 0,   1,-1, 0, 0,-1, 0,  
     1  1, 2, 0,-1, 0, 0,   1, 0, 0, 1, 0, 0,   1, 3, 0, 0, 0, 0,  
     1  1,-2, 2,-1, 0, 0,   1,-2, 0, 1,-1, 0,   1,-3, 2, 0, 0, 0,  
     1  1, 0, 0,-1, 0, 0,   1, 1, 0, 0,-1, 0,   1, 3, 0, 0, 1, 0,  
     1  1, 1,-3, 0, 0, 1,   1,-3, 0, 2, 0, 0,   1, 1, 2, 0, 0, 0,  
     1  1, 0, 0, 1, 1, 0,   1, 2, 0,-1, 1, 0,   1, 0, 2,-1, 0, 0,  
     1  1, 2,-2, 1, 0, 0,   1, 3,-2, 0, 0, 0,   1,-1, 2, 0, 0, 0,  
     1  1, 1, 1, 0, 0,-1,   1, 1,-1, 0, 0, 1,   1, 4, 0,-1, 0, 0,  
     1  1,-4, 2, 1, 0, 0,   1, 0,-2, 1, 0, 0,   1,-2, 2,-1,-1, 0,  
     1  1, 3, 0,-2, 0, 0,   1,-1, 0, 2, 0, 0,   1,-1, 0, 0,-2, 0,  
     1  1, 3, 0, 0, 2, 0,   1,-3, 2, 0,-1, 0,   1, 4, 0,-1, 1, 0,  
     1  1, 0, 0,-1,-1, 0,   1, 1,-2, 0,-1, 0,   1,-3, 0, 2,-1, 0,  
     1  1, 1, 0, 0, 2, 0,   1, 1,-1, 0, 0,-1,   1,-1,-1, 0, 0, 1,  
     1  1, 0, 2,-1, 1, 0,   1,-1, 1, 0, 0,-1,   1,-1,-2, 2, 0, 0,  
     1  1, 2,-2, 1, 1, 0,   1,-4, 0, 3, 0, 0,   1,-1, 2, 0, 1, 0,  
     1  1, 3,-2, 0, 1, 0,   1, 2, 0,-1,-1, 0,   1, 0, 0, 1,-1, 0,  
     1  1,-2, 2, 1, 0, 0,   1, 4,-2,-1, 0, 0,   1,-3, 3, 0, 0,-1,  
     1  1,-2, 1, 1, 0,-1,   1,-2, 3,-1, 0,-1,   1, 0,-2, 1,-1, 0,  
     1  1,-2,-1, 1, 0, 1,   1, 4,-2, 1, 0, 0,   0, 2, 0, 0, 0, 0,  
     1  0, 1, 0,-1, 0, 0,   0, 0, 2, 0, 0, 0,   0, 0, 0, 0, 1, 0,  
     1  0, 2, 0, 0, 1, 0,   0, 3, 0,-1, 0, 0,   0, 1,-2, 1, 0, 0,  
     1  0, 2,-2, 0, 0, 0,   0, 3, 0,-1, 1, 0,   0, 0, 1, 0, 0,-1,  
     1  0, 2, 0,-2, 0, 0,   0, 2, 0, 0, 2, 0,   0, 3,-2, 1, 0, 0,  
     1  0, 1, 0,-1,-1, 0,   0, 1, 0,-1, 1, 0,   0, 4,-2, 0, 0, 0,  
     1  0, 1, 0, 1, 0, 0,   0, 0, 3, 0, 0,-1,   0, 4, 0,-2, 0, 0,  
     1  0, 3,-2, 1, 1, 0,   0, 3,-2,-1, 0, 0,   0, 4,-2, 0, 1, 0,  
     1  0, 0, 2, 0, 1, 0,   0, 1, 0, 1, 1, 0,   0, 4, 0,-2, 1, 0,  
     1  0, 3, 0,-1, 2, 0,   0, 5,-2,-1, 0, 0,   0, 1, 2,-1, 0, 0,  
     1  0, 1,-2, 1,-1, 0,   0, 1,-2, 1, 1, 0,   0, 2,-2, 0,-1, 0/
      k = 0
      nlp = 0
      ndi = 0
      nsd = 0
      ntd = 0
      do 16 ll=1,nin
c  see if Doodson numbers match
      do 14 kk=1,nt
      ii = 0
      do 13 i=1,6
 13   ii = ii + iabs(idd(i,kk)-idtin(i,ll))
      if(ii.eq.0) go to 15
 14   continue
c  have a match - put line into array
 15   if(ii.eq.0.and.k.lt.ncon) then
         k = k + 1
         rl(k) = ampin(ll)*cos(dtr*phin(ll))/abs(tamp(kk))
         aim(k)= ampin(ll)*sin(dtr*phin(ll))/abs(tamp(kk))
C         write(6,*) k,rl(k),aim(k),ll,phin(ll)
c  Now have real and imaginary parts of admittance, scaled by C-E
c  amplitude. Admittance phase is whatever was used in the original
c  expression. (Usually phase is given relative to some reference
c  but amplitude is in absolute units). Next get frequency.
         call tdfrph(idd(1,kk),fr,pr)
         rf(k) = fr
      endif
 16   continue
c  done going through constituents--there are k of them
c  have specified admittance at a number of points. sort these by frequency
c  and separate diurnal and semidiurnal, recopying admittances to get them in
c  order
      call shells(rf,key,k)
      do 21 i=1,k
      if(rf(i).lt.0.5) nlp = nlp + 1
      if(rf(i).lt.1.5.and.rf(i).gt.0.5) ndi = ndi + 1
      if(rf(i).lt.2.5.and.rf(i).gt.1.5) nsd = nsd + 1
      if(rf(i).lt.3.5.and.rf(i).gt.2.5) ntd = ntd + 1
 21   scr(i) = rl(key(i))
      do 23 i=1,k
      rl(i) = scr(i)
 23   scr(i) = aim(key(i))
      do 25 i=1,k
 25   aim(i) = scr(i)
c  now set up splines (8 cases - four species, each real and imag)
c  we have to allow for the case when there are no constituent amplitudes
c  for the long-period or terdiurnal.
      if(nlp.ne.0) call spline(nlp,rf,rl,zdr,scr)
      if(nlp.ne.0) call spline(nlp,rf,aim,zdi,scr)
      call spline(ndi,rf(nlp+1),rl(nlp+1),dr,scr)
      call spline(ndi,rf(nlp+1),aim(nlp+1),di,scr)
      call spline(nsd,rf(nlp+ndi+1),rl(nlp+ndi+1),sdr,scr)
      call spline(nsd,rf(nlp+ndi+1),aim(nlp+ndi+1),sdi,scr)
      if(ntd.ne.0) 
     1   call spline(ntd,rf(nlp+ndi+nsd+1),rl(nlp+ndi+nsd+1),tdr,scr)
      if(ntd.ne.0) 
     1   call spline(ntd,rf(nlp+ndi+nsd+1),aim(nlp+ndi+nsd+1),tdi,scr)
c  evaluate all harmonics using the interpolated admittance
      j = 1
      do 31 i=1,nt
      if((idd(1,i).eq.0.and.nlp.eq.0).or.(idd(1,i).eq.3.and.ntd.eq.0))
     1  go to 31
      call tdfrph(idd(1,i),f(j),p(j))
c  equilibrium phase corrections (including to local phase)
      if(idd(1,i).eq.1) p(j) = p(j) + 90.
      if(idd(1,i).eq.3) p(j) = p(j) + 180.
      sf = f(j)
      if(idd(1,i).eq.0) re = eval(sf,nlp,rf,rl,zdr)
      if(idd(1,i).eq.0) am = eval(sf,nlp,rf,aim,zdi)
      if(idd(1,i).eq.1) re = eval(sf,ndi,rf(nlp+1),rl(nlp+1),dr)
      if(idd(1,i).eq.1) am = eval(sf,ndi,rf(nlp+1),aim(nlp+1),di)
      if(idd(1,i).eq.2) re =
     1   eval(sf,nsd,rf(nlp+ndi+1),rl(nlp+ndi+1),sdr)
      if(idd(1,i).eq.2) am =
     1   eval(sf,nsd,rf(nlp+ndi+1),aim(nlp+ndi+1),sdi)
      if(idd(1,i).eq.3) re =
     1   eval(sf,ntd,rf(nlp+ndi+nsd+1),rl(nlp+ndi+nsd+1),tdr)
      if(idd(1,i).eq.3) am =
     1   eval(sf,ntd,rf(nlp+ndi+nsd+1),aim(nlp+ndi+nsd+1),tdi)
      amp(j) = tamp(i)*sqrt(re**2+am**2)
      p(j) = p(j) + atan2(am,re)/dtr
      j = j + 1
 31   continue
      nout = j - 1
      return
      end
      subroutine tdfrph(idood,freq,phase)
      double precision freq,phase
c
c   Given the Doodson number of a tidal constituent (in idood), returns
c  the frequency and phase.  Phase is returned in degrees and frequency
c  in cycles/day.
c
c   Note that phases must be decreased by 90 degrees if the sum of the order
c  and the species number is odd (as for the 2nd degree diurnals, and 3rd
c  degree low-frequency and semidiurnals).
c   These phases may need further adjustment to allow for the spherical
c  harmonic normalization used; e.g. for that used for the potential by
c  Cartwright and Tayler, 180 degrees must be added for (species,order)
c  = (1,2), (1,3), or (3,3).
c
c  Common block date stores time information, in UT
c
c    calls toymd, leap, juldat, etutc (all for timekeeping)
c
      save itmsave,d,dd
      double precision f1,f2,f3,f4,f5
      double precision fd1,fd2,fd3,fd4,fd5
      double precision d(6),dd(6)
      double precision dayfr,djd,t
      dimension idood(6),itm2(6),itmsave(5)
      common/date/itm(5)
      data itmsave/5*0/
c
c  test to see if time has changed; if so, set the phases and frequencies
c   for each of the Doodson arguments
c
c      intial=0 (corrected 17/12/2007)
      initial=0
      do i=1,5
        if(itm(i).ne.itmsave(i)) initial=1
      enddo
      if(initial.eq.1) then
        do i=1,5
           itmsave(i) = itm(i)
        enddo
c
c convert times to Julian days (UT) then to Julian centuries from J2000.00
c   (ET)
c
        call toymd(itm,itm2)
        jd = juldat(itm2)
        dayfr=  itm(3)/24.d0 + itm(4)/1440.d0 + itm(5)/84600.d0
        year=itm(1)+(itm(2)+dayfr)/(365+leap(itm(1)))
        call etutc(year,delta)
        djd= jd - 0.5d0 + dayfr
        t = (djd - 2451545.0d0 + delta/86400.d0)/36525.d0
c
c IERS expressions for the Delauney arguments, in degrees
c
        f1 =     134.9634025100d0 +
     1    t*( 477198.8675605000d0 +
     1    t*(      0.0088553333d0 +
     1    t*(      0.0000143431d0 +
     1    t*(     -0.0000000680d0 ))))
        f2 =     357.5291091806d0 +
     1    t*(  35999.0502911389d0 +
     1    t*(     -0.0001536667d0 +
     1    t*(      0.0000000378d0 +
     1    t*(     -0.0000000032d0 ))))
        f3 =      93.2720906200d0 +
     1    t*( 483202.0174577222d0 +
     1    t*(     -0.0035420000d0 +
     1    t*(     -0.0000002881d0 +
     1    t*(      0.0000000012d0 ))))
        f4 =     297.8501954694d0 +
     1    t*( 445267.1114469445d0 +
     1    t*(     -0.0017696111d0 +
     1    t*(      0.0000018314d0 +
     1    t*(     -0.0000000088d0 ))))
        f5 =     125.0445550100d0 +
     1    t*(  -1934.1362619722d0 +
     1    t*(      0.0020756111d0 +
     1    t*(      0.0000021394d0 +
     1    t*(     -0.0000000165d0 ))))
c
c  convert to Doodson (Darwin) variables
c
        d(1) = 360.d0*dayfr - f4
        d(2) = f3 + f5
	d(3) = d(2) - f4
        d(4) = d(2) - f1
        d(5) = -f5
        d(6) = d(3) - f2 
c
c   find frequencies of Delauney variables (in cycles/day), and from these
c    the same for the Doodson arguments
c 
        fd1 =  0.0362916471 + 0.0000000013*t
        fd2 =  0.0027377786
        fd3 =  0.0367481951 - 0.0000000005*t
        fd4 =  0.0338631920 - 0.0000000003*t
        fd5 = -0.0001470938 + 0.0000000003*t
        dd(1) = 1.d0 - fd4
        dd(2) = fd3 + fd5
        dd(3) = dd(2) - fd4
        dd(4) = dd(2) - fd1
        dd(5) = -fd5
        dd(6) = dd(3) - fd2 
      endif
c
c   end of intialization (likely to be called only once)
c
c  compute phase and frequency of the given tidal constituent
c
      freq=0.d0
      phase=0.d0
      do i = 1,6
         freq =   freq + idood(i)*dd(i)
         phase = phase + idood(i)*d(i)
      enddo
c adjust phases to fall in the positive range 0 to 360
      phase = dmod(phase,360.d0)
      if(phase.lt.0.d0) phase = phase + 360.d0
      return
      end
      subroutine toymd(it1,it2)
      dimension it1(*),it2(*)
c
c  converts times in it1 given as year and day to year-month-day in it2
c
      idn(m) = mod((367*(m-2-12*((m-14)/12)))/12+29,365)
      mon(jj,m) = (12*(jj-29-m))/367 + 2 + (jj-200)/169
      it2(1)=it1(1)
      it2(2)=mon(it1(2),leap(it1(1)))
      it2(3) = it1(2) - idn(it2(2)) - leap(it2(1))*((9+it2(2))/12)
      return
      end
      function leap(iy)
c
c returns 1 if year is a leap year, by Clavian (Gregorian) rule
c
      leap = 1 - (mod(iy,4)+3)/4
      if(mod(iy,100).eq.0.and.mod(iy,400).ne.0) leap=0
      return
      end
      function juldat(it)
      dimension it(*)
c
c  Julian Date from Gregorian date, Algorithm from p. 604, Explanatory
c    Supplement Amer Ephemeris & Nautical Almanac (cf Comm CACM, 11, 657 (1968)
c    and 15, 918 (1972)) Valid for all positive values of Julian Date
c
      juldat=(1461*(it(1)+4800+(it(2)-14)/12))/4
     1     + (367*(it(2)-2-12*((it(2)-14)/12)))/12
     2     - (3*((it(1)+4900+(it(2)-14)/12)/100))/4+it(3)-32075
      return
      end
      subroutine etutc(year,delta)
c
c   input year (and fraction thereof), 1700-2006
c   output delta = et - utc in seconds
c   tables from p 90, Explanatory Supplement to the A.E., Dr. R A Broucke,
c   JPL, and the leap.sec table in GAMIT
c   utc (and ut) is time usually used (eg in time signals)
c
c
      dimension d(142),tx(39),ty(39),st(23),si(23)
      data d/5.15,4.64,5.36,3.49,3.27,2.45,4.03,1.76,3.3,1.,2.42,.94,2.3
     11,2.27,-.22,.03,-.05,-.06,-.57,.03,-.47,.98,-.86,2.45,.22,.37,2.79
     2,1.2,3.52,1.17,2.67,3.06,2.66,2.97,3.28,3.31,3.33,3.23,3.6,3.52,
     34.27,2.68,2.75,2.67,1.94,1.39,1.66,.88,.33,-.17,-1.88,-3.43,-4.05,
     4-5.77,-7.06,-7.36,-7.67,-7.64,-7.93,-7.82,-8.35,-7.91,-8.03,-9.14,
     5-8.18,-7.88,-7.62,-7.17,-8.14,-7.59,-7.17,-7.94,-8.23,-7.88,-7.68,
     6-6.94,-6.89,-7.11,-5.87,-5.04,-3.9,-2.87,-.58,.71,1.8,3.08,4.63,5.
     786,7.21,8.58,10.5,12.1,12.49,14.41,15.59,15.81,17.52,19.01,18.39,1
     89.55,20.36,21.01,21.81,21.76,22.35,22.68,22.94,22.93,22.69,22.94,2
     93.2,23.31,23.63,23.47,23.68,23.62,23.53,23.59,23.99,23.8,24.2,
     124.99,24.97,25.72,26.21,26.37,26.89,27.68,28.13,28.94,29.42,29.66,
     130.29,30.96,31.09,31.59,32.06,31.82,32.69,33.05,33.16,33.59/
      data tx/61.5,
     262.     ,62.5     ,63.      ,63.5     ,64.      ,64.5     ,65.   ,
     365.5    ,66.      ,66.5     ,67.      ,67.5     ,68.      ,68.25 ,
     468.5    ,68.75    ,69.      ,69.25    ,69.5     ,69.75    ,70.   ,
     570.25   ,70.5     ,70.75    ,71.      ,71.085   ,71.162   ,71.247,
     671.329  ,71.414   ,71.496   ,71.581   ,71.666   ,71.748   ,71.833,
     771.915  ,71.999   ,72.0/
      data ty/33.59,
     234.032  ,34.235   ,34.441   ,34.644   ,34.95    ,35.286   ,35.725,
     336.16   ,36.498   ,36.968   ,37.444   ,37.913   ,38.39    ,38.526,
     438.76   ,39.      ,39.238   ,39.472   ,39.707   ,39.946   ,40.185,
     540.42   ,40.654   ,40.892   ,41.131   ,41.211   ,41.284   ,41.364,
     641.442  ,41.522   ,41.600   ,41.680   ,41.761   ,41.838   ,41.919,
     741.996  ,42.184   ,42.184/
c   update st,si, and nstep to allow for added leap seconds
      data st/1972.5,1973.,1974.,1975.,1976.,1977.,1978.,1979.,
     1        1980.,1981.5,1982.5,1983.5,1985.5,1988.,1990.,
     2        1991.,1992.5,1993.5,1994.5,1996.0,1997.5,1999.0,
     3        2006.0/
      data si/23*1./
      data nstep/23/
      if(year.gt.1972.) go to 6
      if(year.gt.1961.5) go to 2
      if(year.gt.1820.5) go to 1
c  for oldest epochs, use approximation
      if(year.ge.1785.) delta = 6.
      if(year.lt.1785.) delta = (year-1750.)/5.
      if(year.lt.1700.) delta = 0.
      return
c   for 1820.5 to 1961.5, data is spaced at yearly intervals
 1    n = year - 1819.5
      frac = year - (1819.5 + n)
      delta = (d(n+1) - d(n))*frac + d(n)
      return
c   for 1961.5 to 1972.0, interpolate between unequispaced data
 2    do 3 i = 1,38
      if(year-1900..eq.tx(i)) go to 4
      if(year-1900..lt.tx(i)) go to 5
 3    continue
 4    delta = ty(i)
      return
 5    delta=ty(i-1)+(ty(i)-ty(i-1))*((year-1900.-tx(i-1))/(tx(i)-tx(i-1)
     1))
      return
c   after 1972 et-utc has only step offsets. st is the array of step times,
c   si the step sizes (an added second is +1.)
 6    delta = 42.184
      do 7 i = 1,nstep
      if(year.ge.st(i)) delta = delta + si(i)
      if(year.lt.st(i)) return
 7    continue
      return
      end
      subroutine recurs(x,n,hc,nf,om,scr)
      double precision sc,scr
c  does sin and cos recursion to fill in data x, of length n, for
c  nf sines and cosines with frequenciies om (normalized so the
c  nyquist is pi). hc contains alternating cosine and sine coefficients
c  scr is a scratch array of length 3*nf (n.b. - it is double precision)
c
      dimension x(1),hc(1),scr(1),om(1)
c  set up for start of recursion by computing harmonic values
c  at start point and just before it
      do 3 i = 1,nf
      scr(3*i-2) = hc(2*i-1)
      scr(3*i-1) = hc(2*i-1)*cos(om(i)) -hc(2*i)*sin(om(i))
 3    scr(3*i) = 2.*dcos(dble(om(i)))
c  do recursion over data
      do 7 i = 1,n
      x(i) = 0.
c  do recursive computation for each harmonic
      do 5 j  = 1,nf
      x(i) = x(i) + scr(3*j-2)
      sc = scr(3*j-2)
      scr(3*j-2) = scr(3*j)*sc-scr(3*j-1)
 5    scr(3*j-1) = sc
 7    continue
      return
      end

      subroutine shells(x,k,n)
c  sorts an array x, of length n, sorting upward, and returns
c  an array k which may be used to key another array to the
c  sorted pattern (i.e., if we had an array f to which x 
c  corresponded before sorting, then after calling shells,
c  f(k(1)) will be the element of f corresponding to the
c  smallest x, f(k(2)) the next smallest, and so on.
c   revised 29-dec-82 so k is sorted in turn, the values of
c  k that point to identical values of x being put in increasing
c  order
c
c    calls no other routines
c
      dimension x(n),k(n)
      igap = n
      do 1 i = 1,n
 1    k(i) = i
 5    if(igap.le.1) go to 25
      igap = igap/2
      imax = n - igap
 10   iex = 0
      do 20 i = 1,imax
      ipl = i + igap
      if(x(i).le.x(ipl)) go to 20
      sv = x(i)
      ik = k(i)
      x(i) = x(ipl)
      k(i) = k(ipl)
      x(ipl) = sv
      k(ipl) = ik
      iex = iex + 1
 20   continue
      if(iex.gt.0) go to 10
      go to 5
c
c  now sort k's (for identical values of x, if any)
c
 25   j = 1
 30   if(j.ge.n) return
      if(x(j).eq.x(j+1)) go to 33
      j = j + 1
      go to 30
c  have at least two x's with the same value - see how long this is true
 33   l = j
 35   if(x(l).ne.x(l+1)) go to 38
      l = l + 1
      if(l.lt.n) go to 35
c  j and l are the indices within which x(i) does not change - sort k
 38   igap = l - j + 1
 40   if(igap.le.1) j = l + 1
      if(igap.le.1) go to 30
      igap = igap/2
      imax = l-j+1 - igap
 45   iex = 0
      do 50 i=1,imax
      ipl = i + igap + j - 1
      if(k(i+j-1).le.k(ipl)) go to 50
      ik = k(i+j-1)
      k(i+j-1) = k(ipl)
      k(ipl) = ik
      iex = iex + 1
 50   continue
      if(iex.gt.0) go to 45
      go to 40
      end

      subroutine spline(nn,x,u,s,a)
c$$$$ calls no other routines
c  finds array  s  for spline interpolator  eval.
c  nn  number of data points supplied (may be negative, see below)
c  x  array containing x-coordinates where function is sampled.  xx(1),xx(2),...
c     must be a strictly increasing sequence.
c  u  array containing sample values that are to be interpolated.
c  s  output array of 2nd derivative at sample points.
c  a  working space array of dimension at least  nn.
c  if the user wishes to force the derivatives at the ends of the series to     
c  assume specified values, he should put du(1)/dx and du(n)/dx in s1,s2
c  and call the routine with nn=-number of terms in series.  normally a parabola
c  is fitted through the 1st and last 3 points to find the slopes.
c  if less than 4 points are supplied, straight lines are fitted.
      dimension x(*),u(*),s(*),a(*)
c
      q(u1,x1,u2,x2)=(u1/x1**2-u2/x2**2)/(1.0/x1-1.0/x2)
c
      n=iabs(nn)
      if (n.le.3) go to 5000
      q1=q(u(2)-u(1),x(2)-x(1),u(3)-u(1),x(3)-x(1))
      qn=q(u(n-1)-u(n),x(n-1)-x(n),u(n-2)-u(n),x(n-2)-x(n))
      if (nn.gt.0) go to 1000
      q1=s(1)
      qn=s(2)
 1000 s(1)=6.0*((u(2)-u(1))/(x(2)-x(1)) - q1)
      n1= n - 1
      do 2000 i=2,n1
      s(i)= (u(i-1)/(x(i)-x(i-1)) - u(i)*(1.0/(x(i)-x(i-1))+
     + 1.0/(x(i+1)-x(i))) + u(i+1)/(x(i+1)-x(i)))*6.0
 2000 continue
      s(n)=6.0*(qn + (u(n1)-u(n))/(x(n)-x(n1)))
      a(1)=2.0*(x(2)-x(1))
      a(2)=1.5*(x(2)-x(1)) + 2.0*(x(3)-x(2))
      s(2)=s(2) - 0.5*s(1)
      do 3000 i=3,n1
      c=(x(i)-x(i-1))/a(i-1)
      a(i)=2.0*(x(i+1)-x(i-1)) - c*(x(i)-x(i-1))
      s(i)=s(i) - c*s(i-1)
 3000 continue
      c=(x(n)-x(n1))/a(n1)
      a(n)=(2.0-c)*(x(n)-x(n1))
      s(n)=s(n) - c*s(n1)
c  back substitiute
      s(n)= s(n)/a(n)
      do 4000 j=1,n1
      i=n-j
      s(i) =(s(i) - (x(i+1)-x(i))*s(i+1))/a(i)
 4000 continue
      return
c  series too short for cubic spline - use straight lines.
 5000 do 5500 i=1,n
 5500 s(i)=0.0
      return
      end
      function eval(y,nn,x,u,s)
c$$$$ calls no other routines
c  performs cubic spline interpolation of a function sampled at unequally
c  spaced intervals.  the routine spline  should be called to set up the array s
c  y  the coordinate at which function value is desired.
c  nn  number of samples of original function.
c  x  array containing sample coordinates. the sequence x(1),x(2).....x(nn)     
c     must be strictly increasing.
c  u  array containing samples of function at the coordinates x(1),x(2)...      
c  s  array containing the 2nd derivatives at the sample points.  found by the  
c     routine  spline, which must be called once before beginning interpolation.
c  if  y  falls outside range(x(1),x(nn))  the value at the nearest endpoint    
c  of the series is used.
      dimension x(1),u(1),s(1)
      data istart/1/
c
      nn=iabs(nn)
      if (y.lt.x(1))  go to 3000
      if (y.gt.x(nn)) go to 3100
c  locate interval (x(k1),x(k))  containing y
      if (y-x(istart)) 1200,1000,1000
c  scan up the x array
 1000 do 1100 k=istart,nn
      if (x(k).gt.y) go to 1150
 1100 continue
 1150 k1=k-1
      go to 1500
c  scan downwards in x array
 1200 do 1300 k=1,istart
      k1=istart-k
      if (x(k1).le.y) go to 1350
 1300 continue
 1350 k=k1+1
 1500 istart=k1
c  evaluate interpolate
      dy=x(k)-y
      dy1=y-x(k1)
      dk=x(k)-x(k1)
      deli=1./(6.0*dk)
      ff1=s(k1)*dy*dy*dy
      ff2=s(k)*dy1*dy1*dy1
      f1=(ff1+ff2)*deli
      f2=dy1*((u(k)/dk)-(s(k)*dk)/6.)
      f3=dy*((u(k1)/dk)-(s(k1)*dk)/6.0)
      eval=f1+f2+f3
      return
c  out of range.  substitute end values.
 3000 eval=u(1)
      return
 3100 eval=u(nn)
      return
      end
