#!/usr/bin/perl
no warnings 'deprecated';

$,=" ";
$[=1;
$\="\n";
$|=1;

if ( $#ARGV == 0 ) {
print "this script converts a rinex file into a table file by executing:

$0 rinex_file [-r rcv_name] [-e LXX LYY LZZ]
or 
cat rinex_file | rnx2ti.perl - [-r rcv_name] [-e LXX LYY LZZ]

The script indentifies the rinex version and the GNSS systems, and writes into
the standard input the following information (one register for each satellite and epoch):\n
#field			What?					Units
1			year				
2			day of year				
3			epoch time				seconds
4			receiver name
5			GNSS system				G, E
6			Satellite Id.
7			L1					meters
8			L2, L7					meters
9			L5					meters
10			P1					meters
11			P2, P7					meters
12			P5					meters
14			LLI
13			S/N L1
14			S/N L2, L7
15			S/N L5

Among the standard input, a log file is also generated containing 
iformation about the rinex file (receiver, antenna ....)
If no P1 observations are present, P1 is generated from C1 by correcting
the DCB.
";
goto end;
                   }


##################CONSTANTS
$c_light=299792458;
$f0=10.23e6;
$f[1]=1575.42e6;
$f[2]=1227.60e6;
$f2_gps=1227.60e6;
$f[5]=1176.45e6;
$f[6]=1278.75e6;
$f[7]=1207.140e6;
$f[8]=1191.795e6;

##############
##### OPEN THE LOG FILE
open (F10,"> rnx2txt.log");
###########################

######### 
if ($ARGV[1] eq "-") {
     open (F1,"cat|") or die "no input data";
     printf (F10 "FILE : STDIO\n");
                     } else {
     open (F1,"$ARGV[1]")  or die "file $ARGV[1] don't exist";
     printf (F10 "FILE : %s\n",$ARGV[1]);
                            };

shift @ARGV;

while (@ARGV) {
if ($ARGV[1] eq "-r") {$rcv_name = $ARGV[2];shift @ARGV;shift @ARGV};
if ($ARGV[1] eq "-e") {
shift @ARGV;
$i_car=0;
while (substr($ARGV[1],1,1) ne "-" && @ARGV) {
$i_car++;
$carriers[$i_car]=$ARGV[1];
$carrier_type{substr($ARGV[1],2,1)}=substr($ARGV[1],3.1);
shift @ARGV;
                                              }
                      }
               }

#####################################

###########################

while (<F1>) {
chop;
@d=split(' ',$_,50);
###############################READING THE RINEX HEADER #############
###### IDENTIFY RINEX VERSION
if (substr($_,61,13) eq "RINEX VERSION") {$rinex_version=substr($_,6,1)*1;printf (F10 "RINEX VERSION : %1d \n",$rinex_version);$end_of_header=0;$n_sat=0}

#############################

###### IDENTIFY MARKER NAME
if (substr($_,61,11) eq "MARKER NAME" && length($rcv_name)<2) {$rcv_name=substr($_,1,4);}

#############################

##### IDENTIFY RCV TYPE
if (substr($_,61,12) eq "REC # / TYPE") {$rcv_type=substr($_,21,20);printf (F10 "RCV TYPE : %20s \n",$rcv_type)}

############################

##### IDENTIFY ANTENNA TYPE
if (substr($_,61,12) eq "ANT # / TYPE") {$ant_type=substr($_,21,20);printf (F10 "ANTENNA TYPE : %20s \n",$ant_type)}

############################

##### TIME OF FIRST OBS

if (substr($_,61,17) eq "TIME OF FIRST OBS") {$year=$d[1];$yy=substr($year,3,2);$month=$d[2];$day=$d[3];printf (F10 "YY TIME OF FIRST OBS : %2s %30s\n",$yy,substr($_,1,30))}

#### ARP vector
if (substr($_,61,16) eq "ANTENNA: DELTA H") {@ant_vector=split(' ',$_,3);;printf (F10 "ANTENNA ARP VECTOR : %7.4f %7.4f %7.4f \n",@ant_vector)}
################

#### APPROX POSITION XYZ
if (substr($_,61,15) eq "APPROX POSITION") {@rcv_pos=split(' ',$_,3);printf (F10 "RCV POSITION : %14.4f %14.4f %14.4f \n",@rcv_pos)}
#####################

#################### IDENTIFY OBSERVATIONS
if ($rinex_version==2) {
if (substr($_,61,19) eq "# / TYPES OF OBSERV" && $n_obs==0) {
$n_obs=@d[1];
if ($n_obs <= 9) {
for $i (1..$n_obs) {$obs{substr($d[$i+1],1,2)}=$i}
                 }
if ($n_obs > 9 && $n_obs <= 18) {
for $i (1..9) {$obs{substr($d[$i+1],1,2)}=$i}
$s_=<F1>;
@ss_=split(" ",$s_,10);
for $i (10..$n_obs) {$obs{substr($ss_[$i-9],1,2)}=$i}
                                }

if ($n_obs > 18 && $n_obs <=27) {
for $i (1..9) {$obs{substr($d[$i+1],1,2)}=$i}
$s_=<F1>;
@ss_=split(" ",$s_,10);
for $i (10..18) {$obs{substr($ss_[$i-9],1,2)}=$i}
$s_=<F1>;
@ss_=split(" ",$s_,10);
for $i (19..$n_obs) {$obs{substr($ss_[$i-18],1,2)}=$i}

                                }
$n_lines=int(($n_obs-1)/5)+1;
printf (F10 "# AND TYPE OF OF OBSERVATIONS : %s\n",substr($_,1,60));
                                                             }
                       }


if ($rinex_version==3) {
if (substr($_,61,19) eq "SYS / # / OBS TYPES" && $n_obs{$d[1]}==0) {
$n_obs{$d[1]}=$d[2];
$n_obs_lines{$d[1]}=int(($d[2]-1)/13)+1;
$ii=0;

for $line (1..$n_obs_lines{$d[1]}) {

if ($line == 1) {
         if ($d[1] eq "E") {$carrs=()}
	if ($n_obs_lines{$d[1]}==1) {

	for $j (1..$n_obs{$d[1]}) {
            $ii++;
            if ($d[1] eq "G" || $#carriers==0 || $carrier_type{substr($d[$j+2],2,1)} eq substr($d[$j+2],3,1)) {
            $obs{$d[1]." ".$d[$j+2]}=$ii;
        if ($d[1] eq "E" && substr($d[$j+2],1,1) eq "L") {push(@carrs,$d[$j+2])}
                              }
                                  }

        	                    } else {
	for $j (1..13) {
         $ii++;

            if ($d[1] eq "G" || $#carriers==0 || $carrier_type{substr($d[$j+2],2,1)} eq substr($d[$j+2],3,1)) {
         $obs{$d[1]." ".$d[$j+2]}=$ii;
        if ($d[1] eq "E" && substr($d[$j+2],1,1) eq "L") {push(@carrs,$d[$j+2])}
                               }
                                            }
                                  }
                } else {

@c=split(' ',<F1>,15);
for $j (1..13) {
     $ii++;
            if ($d[1] eq "G" || $#carriers==0 || $carrier_type{substr($c[$j],2,1)} eq substr($c[$j],3,1)) {
     $obs{$d[1]." ".$c[$j]}=$ii;
          if ($d[1] eq "E" && substr($c[$j],1,1) eq "L") {push(@carrs,$c[$j])}
                              }
               }
                       }
                            }
                                                                   }
                       }
############################################
if (substr($_,61,13) eq "END OF HEADER") {
$end_of_header=1;

####### CHECK FOR A RECEIVER NAME
if (length($rcv_name) < 4) {print "WARNING : NO RCV NAME HAS BEEN PROVIDED";goto end} else {printf (F10 "MARKER NAME : %4s \n",$rcv_name)};

######## TEST THE P1 OBSERVATION for DCB corrections (only for GPS)

foreach $key (keys(%obs)) {
if (substr($key,1,2)  eq "P1") {$p1_present=1}
                          }
if ($p1_present==0) {
printf (F10 "P1 PRESENT : NO\n");
print (F10 "\n WARNING A.I.: It is necessary to correct from DCB");

if ($rinex_version==2) {$obs{"P1"}=$obs{"C1"}};
if ($rinex_version==3) {
foreach $label ( keys %obs ) {
@c=split(' ',$label,4);
if (substr($c[2],3,1) eq "C" && substr($c[2],1,1) eq "C" ) {
              $obs{$c[1]." C".substr($c[2],2,1)."P"}=$obs{$label}
                                                           }
                             }
                       }
                    } else {
printf (F10 "P1 PRESENT : YES\n")
                           }

######################### 
#### reads the CA2P1 corrections indepentely of P1 present
########################
######## searchs the receiver in ../info_files/GPS_Receiver_Types.gz
$rr=`zgrep "$rcv_type" ../info_files/GPS_Receiver_Types.gz`;
$c12p1_type=substr($rr,22,1);
print (F10 "RECEIVER CA2P1 TYPE : ",$c12p1_type);

##################################
$c12p1_file="CA2P1.dat";
print (F10 "CA2P1 file : ",$c12p1_file);
open (F3,$c12p1_file);
while(<F3>) {
chop;
@bb=split(' ',$_,5);
$c12p1_corr{$bb[2]*1}=-1*$bb[4];
            }
	    
close (F3);
################################################


################################################

##### Write the different obsevations in the log file
##### 
printf (F10 "OBSERVATION TYPES :\n");
foreach $key ( sort{$a<=>$b} (keys %obs) ) {
if (($obs{$key} <= $n_obs{substr($key,1,1)} && $rinex_version==3) || $rinex_version==2) {
printf (F10 "%02d %5s\n",$obs{$key},$key);
         }
if ($rinex_version==3) {
$obs_lab=substr($key,1,4);
if ($obs{$obs_lab} && substr($key,3,1) eq "L" && $obs_lab ne $key) {
print "There exist several posibilities for ",$obs_lab," you must select one of them in the command line";
$select_obs=1;
                                                                    } else {

$obs{$obs_lab}=$obs{$key};
                                                     }
                       }

####### end foreach $key ( sort{$a<=>$b} (keys %obs) )
                                           }

if ($select_obs==1) {goto end}
                                         }

################# SORTING  THE CARRIER PHASES BY ITS FREQ
$carr_list=@carrs;
if ($#carrs==2) {
for $i (1..2) {if (substr($carrs[$i],2,1)*1==1 ) {$carr_list[1]=$carrs[$i]} else {$carr_list[2]=$carrs[$i]}
              }
                }

if ($#carrs==3) {
$freq_min=1e10;
for $i (1..3) {
$ifreq=substr($carrs[$i],2,1)*1;
if ($ifreq==1 ) {$carr_list[1]=$carrs[$i]} ;
if ($f[$ifreq]<$freq_min) {$freq_min=$f[$ifreq];$i_min=$i}
              }
$carr_list[3]=$carrs[$i_min];
$ifreq=substr($carrs[$i],2,1)*1;
for $i (1..3) {if ($i!=1 && $i!=$i_min) {$carr_list[2]=$carrs[$i]}}
                }



###################################################################
###############################START READING THE RINEX BODY #######
		#
		#$old_fh = select(STDOUT);
		#$| = 1;
		#select($old_fh);
close(F10);

################### READING THE TIME RECORD
if ($end_of_header==1) {

##### Rinex version 2
if ($rinex_version==2 && $#d>6) {
cal2doy(@d[1..6]);
$epoch_flag=substr($_,29,1);
$n_sat=substr($_,30,3)*1;

#### LEAP MSEC
$leap=-1*($d[6]+$d[5]*60+$d[4]*3600-$sec)*$c_light;
#############
$ss_=$_;
$i_0=0;
for $i (1..$n_sat) {
$i_0++;
if ($i==13) {
$ss_=<F1>;
$i_0=$i-12;
             }
if ($i==25) {
$ss_=<F1>;
$i_0=$i-24;
             }

$gnss_type_[$i]=substr($ss_,33+($i_0-1)*3,1);
if ($gnss_type_[$i] eq " ") {$gnss_type_[$i]="G"}
$sat_[$i]=substr($ss_,34+($i_0-1)*3,2);
                    }
                                }
#########################

##### Rinex version 3
if ($rinex_version==3 && $d[1] eq ">") {
cal2doy(@d[2..7]);
$epoch_flag=substr($_,32,1);
$n_sat=substr($_,33,3)*1;

#### LEAP MSEC
$leap=-1*($d[7]+$d[6]*60+$d[5]*3600-$sec)*$c_light;
                                       }

@ggg=convert_time_gLAB($year,$doy);
$gps_week=$ggg[1];
$gps_day=$ggg[2];

#################### Glonass
if ( $ndata == 0 ) {
open (F11,"../../info_files/glonass.status");
while (<F11>) {
@glo=split(" ",$_,5);
if ($year+$doy/365>$glo[1] && $year+$doy/365<$glo[2]) {
$glo_chan{1*$glo[3]}=$glo[4];
$e_glo_chan{1*$glo[3]}=1;
                                                      }

              }
                   }
close(F11);
$ndata++;




######################################

####################
##########################################

################## READING THE OBSERVATION RECORDS

for $isat (1..$n_sat) {

##### rinex version 2
if ($rinex_version==2) {
for $ii (1..$n_lines) {
$c[$ii]=<F1>;
                      }
$lli=0;
$freq_flag="1";
$pos_i=int(($obs{"L1"}-1)/5)+1;
$pos_j=$obs{"L1"}-5*($pos_i-1);
$l1=substr($c[$pos_i],($pos_j-1)*16+1,14);
$lli+=substr($c[$pos_i],($pos_j-1)*16+15,1)%2;
$sn1=substr($c[$pos_i],($pos_j-1)*16+16,1);

if ($obs{"L2"}) {
$freq_flag=$freq_flag."2";
$pos_i=int(($obs{"L2"}-1)/5)+1;
$pos_j=$obs{"L2"}-5*($pos_i-1);
$l2=substr($c[$pos_i],($pos_j-1)*16+1,14);
$lli+=substr($c[$pos_i],($pos_j-1)*16+15,1)%2;
$sn2=substr($c[$pos_i],($pos_j-1)*16+16,1);
                 }
if ($obs{"S1"}) {
$pos_i=int(($obs{"S1"}-1)/5)+1;
$pos_j=$obs{"S1"}-5*($pos_i-1);
$sn1=int(10*substr($c[$pos_i],($pos_j-1)*16+1,14));
                }

if ($obs{"S2"}) {
$pos_i=int(($obs{"S2"}-1)/5)+1;
$pos_j=$obs{"S2"}-5*($pos_i-1);
$sn2=int(10*substr($c[$pos_i],($pos_j-1)*16+1,14));
                }

$p1_pr=$p1_present;

$pos_i=int(($obs{"P1"}-1)/5)+1;
$pos_j=$obs{"P1"}-5*($pos_i-1);
$p1=substr($c[$pos_i],($pos_j-1)*16+1,14);
$pos_i=int(($obs{"C1"}-1)/5)+1;
$pos_j=$obs{"C1"}-5*($pos_i-1);
$c1=substr($c[$pos_i],($pos_j-1)*16+1,14);

if ($p1*1==0) {
$pos_i=int(($obs{"C1"}-1)/5)+1;
$pos_j=$obs{"C1"}-5*($pos_i-1);
$p1=substr($c[$pos_i],($pos_j-1)*16+1,14);
$p1_pr=0;
$c12p1_type=2;
              }

$pos_i=int(($obs{"P2"}-1)/5)+1;
$pos_j=$obs{"P2"}-5*($pos_i-1);
$p2=substr($c[$pos_i],($pos_j-1)*16+1,14);


$gnss_type=$gnss_type_[$isat];
$sat=$sat_[$isat];

if ($gnss_type eq "G") {
if ($p1_pr==0 && $c12p1_type==1) {
                               $p1=$p1+$c12p1_corr{$sat*1};
                               $p2=$p2+$c12p1_corr{$sat*1};
                                  }
if ($p1_pr==0 && $c12p1_type==2) {
                              $p1=$p1+$c12p1_corr{$sat*1};
                                  }
                        }

$dop1=99999.9999;
$dop2=99999.9999;
if ($obs{"D1"}) {
$pos_i=int(($obs{"D1"}-1)/5)+1;
$pos_j=$obs{"D1"}-5*($pos_i-1);
$dop1=substr($c[$pos_i],($pos_j-1)*16+1,14);
               }
if ($obs{"D2"}) {
$pos_i=int(($obs{"D2"}-1)/5)+1;
$pos_j=$obs{"D2"}-5*($pos_i-1);
$dop2=substr($c[$pos_i],($pos_j-1)*16+1,14);
               }


$ff1=$f[1];
$ff2=$f[2];

if ($gnss_type eq "C") {
          $ff1=1561.098e6;
          $ff2=1268.52e6;
                       }
if ($gnss_type eq "R") {
          $ff1=1602e6+$glo_chan{1*$sat}*562.5e3;
          $ff2=1246e6+$glo_chan{1*$sat}*437.5e3;
                       }

if ( $p1*$p2> 1.e4) {
$c1_p1=$c1-$p1;
if ($p1_pr==0) {$c1_p1=99999}
printf "%2d %3d %6d %4s %3s  %2d %16.4f %16.4f %16.4f %14.4f %1d %3d %3d %5d %1d\n",$year,$doy,$sec,$rcv_name,$gnss_type."12",$sat,$l1*$c_light/$ff1+$leap,$l2*$c_light/$ff2+$leap,$p1*1+$leap,$p2*1+$leap,$lli,$sn1,$sn2,$gps_week,$gps_day;
                  }

                       }
#####################

##### rinex version 3
if ($rinex_version==3) {
$l1=0;$l2=0;$l5=0;$p1=0;$p2=0;$p5=0;$lli=0;;
$c=<F1>;
$gnss_type=substr($c,1,1);
$sat=substr($c,2,2);

if ($gnss_type eq "G") { 
$freq_flag="1";
$f_[1]=$f[1];
$l_[1]=substr($c,4+($obs{$gnss_type." L1C"}-1)*16,14);
$p_[1]=substr($c,4+($obs{$gnss_type." C1P"}-1)*16,14);
$dop_[1]=substr($c,4+($obs{$gnss_type." D1P"}-1)*16,14);
$lli+=substr($c,4+($obs{$gnss_type." L1C"}-1)*16+14,1)%2;
$sn_[1]=substr($c,4+($obs{$gnss_type." L1C"}-1)*16+15,1);
if ($obs{$gnss_type." L2P"}) {
$f_[2]=$f[2];
$freq_flag=$freq_flag."2";
$l_[2]=substr($c,4+($obs{$gnss_type." L2P"}-1)*16,14);
$p_[2]=substr($c,4+($obs{$gnss_type." C2P"}-1)*16,14);
$dop_[2]=substr($c,4+($obs{$gnss_type." D2P"}-1)*16,14);
$lli+=substr($c,4+($obs{$gnss_type." L2P"}-1)*16+14,1)%2;
$sn_[2]=substr($c,4+($obs{$gnss_type." L2P"}-1)*16+15,1);
                             }
if ($obs{$gnss_type." L5P"}) {
$f_[2]=$f[5];
$freq_flag=$freq_flag."5";
$l_[2]=substr($c,4+($obs{$gnss_type." L5P"}-1)*16,14);
$p_[2]=substr($c,4+($obs{$gnss_type." C5P"}-1)*16,14);
$dop_[2]=substr($c,4+($obs{$gnss_type." D5P"}-1)*16,14);
$lli+=substr($c,4+($obs{$gnss_type." L5P"}-1)*16+14,1)%2;
$sn_[2]=substr($c,4+($obs{$gnss_type." L5P"}-1)*16+15,1);
                             }

if ($p1_present==0 && $c12p1_type==1) {
                               $p_[1]=$p_[1]+$c12p1_corr{$sat*1};
                               $p_[2]=$p_[2]+$c12p1_corr{$sat*1};
                                      }
if ($p1_present==0 && $c12p1_type==2) {
                               $p_[1]=$p_[1]+$c12p1_corr{$sat*1};
                                      }

                       }
if ($gnss_type eq "E") { 

$freq_flag="";
for $i_carr (1..$#carr_list) {
$i_freq=substr($carr_list[$i_carr],2,1);
$freq_flag=$freq_flag.$i_freq;
$p_lab="C".$i_freq.substr($carr_list[$i_carr],3,1);
$d_lab="D".$i_freq.substr($carr_list[$i_carr],3,1);
$l_[$i_carr]=substr($c,4+($obs{$gnss_type." ".$carr_list[$i_carr]}-1)*16,14);
$sn_[$i_carr]=substr($c,4+($obs{$gnss_type." ".$carr_list[$i_carr]}-1)*16+15,1);
$lli+=substr($c,4+($obs{$gnss_type." ".$carr_list[$i_carr]}-1)*16+14,1)%2;
$p_[$i_carr]=substr($c,4+($obs{$gnss_type." ".$p_lab}-1)*16,14);
$f_[$i_carr]=$f[$i_freq];
$dop_[$i_carr]=substr($c,4+($obs{$gnss_type." ".$d_lab}-1)*16,14);
                             }


                       }

if (length($freq_flag)==1 && $p_[1]>1) {
printf "%2d %3d %6d %4s %4s  %2d %16.4f %14.4f %1d %1d\n",$year,$doy,$sec,$rcv_name,$gnss_type.$freq_flag,$sat,$l_[1]*$c_light/$f_[1]+$leap,$p_[1]*1+$leap,$lli,$sn_[1];
                                        }

if (length($freq_flag)==2 && ($p_[2]*$p_[1])**2>1) {
printf "%2d %3d %6d %4s %4s  %2d %16.4f %16.4f %14.4f %14.4f %1d %1d %1d %14.4f %14.4f\n",$year,$doy,$sec,$rcv_name,$gnss_type.$freq_flag,$sat,$l_[1]*$c_light/$f_[1]+$leap,$l_[2]*$c_light/$f_[2]+$leap,$p_[1]*1+$leap,$p_[2]*1+$leap,$lli,$sn_[1],$sn_[2],$dop_[1]*$c_light/$f_[1],$dop_[2]*$c_light/$f_[2];
                                                   }

if (length($freq_flag)==3 && ($p_[3]*$p_[2]*$p_[1])**2>1) {
printf "%2d %3d %6d %4s %4s  %2d %16.4f %16.4f %16.4f %14.4f %14.4f %14.4f %1d %1d %1d %1d %14.4f %14.4f %14.4f\n",$year,$doy,$sec,$rcv_name,$gnss_type.$freq_flag,$sat,$l_[1]*$c_light/$f_[1]+$leap,$l_[2]*$c_light/$f_[2]+$leap,$l_[3]*$c_light/$f_[3]+$leap,$p_[1]*1+$leap,$p_[2]*1+$leap,$p_[3]*1+$leap,$lli,$sn_[1],$sn_[2],$sn_[3],$dop_[1]*$c_light/$f_[1],$dop_[2]*$c_light/$f_[2],$dop_[3]*$c_light/$f_[3];

                                                           }


                       }
#####
############ end of for $i (1..$n_sat)
                   }


#### end of reading rinex body
                       }
              }

close (F10);
$logfile=$rcv_name."_rnx.log";
exec("cp rnx2txt.log $logfile");
end:;
################################### Sub CAL2DOY
## transform "year month day hour min sec" into "yy day_of_year sec_of_day"
######################################
sub cal2doy {
local (@xy) = @_;
my @month_days0=(31,28,31,30,31,30,31,31,30,31,30,31);
$year=$xy[1]%100;$month=$xy[2];$day=$xy[3];$hour=$xy[4];$min=$xy[5];
$sec=$xy[6];
$doy=0;
@month_days=@month_days0;


#### remember that year 2000 had also 366 days, this must be corrected by the
#### XXII century.
if ($year%4==0) {$month_days[2]=29}
for ($i=1;$i<=$month-1;$i++){$doy=$doy+$month_days[$i]}
$doy=$doy+$day;
$sec=int($sec+0.3)+$hour*3600+$min*60;
            }

################################### Sub Convert time
# computes the GPS week (taken from rnx2txt.pl
######################################
sub convert_time {
local (@xy) = @_;
$year_perl=$xy[1];
if ($year_perl< 100 && $year_perl>80) {$year_perl=$year_perl+1900}
if ($year_perl< 100 && $year_perl<80) {$year_perl=$year_perl+2000}
$yy_perl=($year_perl-1900)%100;
if ($yy_perl<10) {$yy_perl="0".$yy_perl}

$doy_perl=$xy[2];

$date0_perl=$yy_perl."-01-01";

@time0_perl=gmtime(0);
$date0_perl=$time0_perl[6]."-01-01";

$d_doys_perl=($sec_secy_perl-$sec_gps0_perl)/86400.+$doy_perl-1;
$gps_week_perl=int($d_doys_perl/7.)+1;
$gps_day_perl=int($d_doys_perl-7*($gps_week_perl-1));

$sec_system_perl=$sec_gps0_perl-$sec_system_perl+$d_doys_perl*86400;
@time1_perl=gmtime($sec_system_perl);

$time00_perl=time();
return ($gps_week_perl,$gps_day_perl);
}

sub convert_time_gLAB {
local (@xy) = @_;
$year_gLAB=$xy[1];
$doy_gLAB=$xy[2];
if( $year_gLAB<=70) {
	$year_gLAB=$year_gLAB+2000;
} elsif ($year_gLAB<100 && $year_gLAB>70) {
	$year_gLAB=$year_gLAB+2000;
}
$tmp_gLAB=int(365.25*($year_gLAB-1))+428+$doy_gLAB;
$MJDN_gLAB=$tmp_gLAB+1720981.5-2400000.5;
$gpsday_gLAB=$MJDN_gLAB-44244;
$weekday_gLAB=$gpsday_gLAB%7;
$GPSweek_gLAB=int($gpsday_gLAB-($gpsday_gLAB%7))/7;

return ($GPSweek_gLAB,$weekday_gLAB);
}
