c234567
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
CC THIS PROGRAM COMPUTES THE LUNI-SOLAR TIDE DISPLACEMENT
cc the inputs are: iyear,idoy,imonth,iday,sec,y1,y2,y3
cc where y1,y2,y3 are the rcv coordinates in kilometers
cc
cc The outputs are the second of day and the displacement 
cc (XYZ) in meters in the WGS84.
ccccccccccccccccccccccccccccccccccccccccccccccccccccccc
       implicit double precision (a-h,o-z)

        dimension rp(3),rpp(3),delta_sun(3),delta_moon(3),
     * delta(3),r_sun(3),r_moon(3)
        character*2 cts
c       ..................................
c       Love numbers
        xh_lov=0.609d0
        xl_lov=0.0852d0
c       Ratio of mass Sun to Earth
        xmu_s=332946.0d0
c       Ratio of mass Moon to Earth
        xmu_m=0.01230002d0
c       acceleration gravity at Earth surfice (m/s2)
        grav=9.80665d0
        g2r=datan2(1.d0,1.d0)/45.d0
        
        ph_lag=0.d0
c       ..................................

100     read (*,*,end=101) iyear,idoy,imonth,iday,sec,y1,y2,y3
       rpp(1)=y1*1.d3
       rpp(2)=y2*1.d3
       rpp(3)=y3*1.d3

       call rot3(ph_lag,rpp,rp)

        day_frac=sec/86400.d0

        if (iyear .lt. 80) then
        iiyear=iyear+2000
         elseif (iyear .lt. 100) then
        iiyear=iyear+1900
          elseif (iyear .gt. 100) then
         iiyear=iyear
         endif


       call sub_sun_moon(iiyear,idoy,day_frac,xsun,ysun,zsun,
     * xmoon,ymoon,zmoon)

         
c        ...From Km to m...
         xsun=xsun*1.d3
         ysun=ysun*1.d3
         zsun=zsun*1.d3

         xmoon=xmoon*1.d3
         ymoon=ymoon*1.d3
         zmoon=zmoon*1.d3

         r_sun(1)=xsun*1.d0
         r_sun(2)=ysun*1.d0
         r_sun(3)=zsun*1.d0

         r_moon(1)=xmoon*1.d0
         r_moon(2)=ymoon*1.d0
         r_moon(3)=zmoon*1.d0
c        ..................

        rpRs=rp(1)*xsun+rp(2)*ysun+rp(3)*zsun
        Rs2=xsun**2+ysun**2+zsun**2
        rp2=rp(1)**2+rp(2)**2+rp(3)**2
        xy2p=rp(1)**2+rp(2)**2

        xfac_s=3.d0*xmu_s*rp2/(dsqrt(Rs2)**5)

        g1sun=xfac_s*(rpRs**2/2.d0-rp2*Rs2/6.d0)
        g2sun=xfac_s*rpRs*(ysun*rp(1)-xsun*rp(2))*
     *        dsqrt(rp2/xy2p)
        g3sun=xfac_s*rpRs*(dsqrt(xy2p)*zsun -
     *   rp(3)/dsqrt(xy2p)*(rp(1)*xsun+rp(2)*ysun))

c       ....g_moon.........................................
        rpRm=rp(1)*xmoon+rp(2)*ymoon+rp(3)*zmoon
        Rm2=xmoon**2+ymoon**2+zmoon**2
        rp2=rp(1)**2+rp(2)**2+rp(3)**2
        xy2p=rp(1)**2+rp(2)**2

        xfac_m=3.d0*xmu_m*rp2/(dsqrt(Rm2)**5)

        g1moon=xfac_m*(rpRm**2/2.d0-rp2*Rm2/6.d0)
        g2moon=xfac_m*rpRm*(ymoon*rp(1)-xmoon*rp(2))*
     *         dsqrt(rp2/xy2p)
        g3moon=xfac_m*rpRm*(dsqrt(xy2p)*zmoon -
     *   rp(3)/dsqrt(xy2p)*(rp(1)*xmoon+rp(2)*ymoon))
c      ....................................................

c      ...Solid-tidal displacement in a local Cartesian frame (uen)...
        delta_sun(1)=xh_lov*g1sun
        delta_sun(2)=xl_lov*g2sun
        delta_sun(3)=xl_lov*g3sun

        delta_moon(1)=xh_lov*g1moon
        delta_moon(2)=xl_lov*g2moon
        delta_moon(3)=xl_lov*g3moon

        do i=1,3
         delta(i)=(delta_sun(i)+delta_moon(i))*1.d-3
        enddo
c      ...............................................................

         xlon=datan2(y2,y1)
         xlat=datan2(y3,dsqrt(y1**2+y2**2))

          call rot2(xlat,delta,rp)
          call rot3(-xlon,rp,delta)

c         imonth=12
c         iday=16
          fhr=sec/3600.d0


          call tide(rpp,iiyear,imonth,iday,fhr,r_sun,r_moon,delta)

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
        write (*,'(i6,1x,3(f7.4,1x))') nint(sec),delta
         goto 100

101    continue

       end




c --------------------------------------------------------------


        subroutine sub_sun_moon(iyear,idoy,fracday,
     *                            xs,ys,zs,xm,ym,zm)

        implicit double precision (a-h,o-z)
        dimension PV(6)

c   -------------------------------------------------------
c       Coordenadas Topocentricas "aproximadas" del SOL
c       y la LUNA en el sistema CTS respecto al ecuador
c       medio de la epoca. Tambien calcula el tiempo
c       sidereo y la fecha juliana modificada (JD-2400000.5)
c
c
c
c iyear idoy frac_day ---> |sun_moon| ---> xs ys zs xm ym zm
c                                                     (Km)
c

c           echo 1982 1 0.5 | sun_moon
c           ====>
c            r_Sun   27462788.4 -132585931. -57489356.9
c                   135384031.  2097143.89 -57489356.9  18.7209897
c              MJD  44970.5
c            r_Moon  378082.745 -50513.2178 -54533.2007
c                    120557.899  361889.404 -54533.2007  18.7209897
c
c                 (MJD:  44970.5, sid: 18.7209897 )
c
c  ---------------------------------------------------------


c       ....AU in Km...
        AU=1.49597870D8
c       ...............

        IY=iyear
        ID=idoy
        fr=fracday

c       print *,"WWS", IY, ID, fr
        call sla_EARTH (IY, ID, fr, PV)

c       print *, "r_Sun", -PV(1)*AU,-PV(2)*AU,-PV(3)*AU
        xs=-PV(1)*AU
        ys=-PV(2)*AU
        zs=-PV(3)*AU

        UT=fr*24.d0
c       print *,xs,ys,zs

c       Coordenadas Topocentricas del SOL respecto
c       al ecuador y el meridiano de Greenwich de la
c       epoca (CTS) (Transformacion aproximada, solo se considera
c       la rotacion del t_sidereo, no se considera el
c       movimiento  del polo, ni precesion, ni nutacion,...)

        call sub_eq2wgs_yydd(iy,id,UT,xs,ys,zs,sid)

c        print *, xs,ys,zs,sid


        call  yydoy2MJD(iy,id,fr,xmjd)
c        print *, "MJD",xmjd

        call sla_DMOON (xmjd, PV)
c        print *, "r_Moon", PV(1)*AU,PV(2)*AU,PV(3)*AU
        xm=PV(1)*AU
        ym=PV(2)*AU
        zm=PV(3)*AU

c       Coordenadas Topocentricas de la LUNA respecto
c       al ecuador y el meridiano de Greenwich de la
c       epoca (CTS) (Transformacion aproximada, solo se considera
c       la rotacion del t_sidereo, no se considera el
c       movimiento  del polo, ni precesion, ni nutacion,...)
        call sub_eq2wgs_yydd(iy,id,UT,xm,ym,zm,sid)

c        print *,'LUNA',xm,ym,zm,sid,xs

        return
        end


        subroutine sub_eq2wgs_yydd(iy,idoy,UT,x,y,z,sid)
        implicit double precision (a-h,o-z)
        dimension r(3), rp(3)

c -------------------------------------------------------------------
c
c       (iy idoy UT,x,y,z,sid)  ---->|eq2wgs_yydd| ---> (x,y,z,sid)
c      coordenadas ecuatoriales                          WGS84
c         x => Aries                                  x => Greenwich
c         z => Polo Norte                             z => Polo Norte
c
c                                                  sid= tiempo sidereo
c                                                       para (iy idoy UT)
c --------------------------------------------------------------------
c
c
c      @gAGE (grup d'Astronomia i Geodesia Espacial). UPC
c --------------------------------------------------------------------



        pi=datan2(1.d0,1.d0)*4

        r(1)=x
        r(2)=y
        r(3)=z


c       Calculo del tiempo sidereo a 0UT del dia del anyo
        call yyddUT2sid(iy,idoy,UT,sid)

        ts=sid*pi/12.d0

c       Transformacion de coordenadas
        call rot3(ts,r,rp)

        x=rp(1)
        y=rp(2)
        z=rp(3)

c       print *, ts,rp
c        write(*,'(f12.5,1x,f12.5,1x,f12.5)') rp

        return
        end
      SUBROUTINE sla_EARTH (IY, ID, FD, PV)
*+
*     - - - - - -
*      E A R T H     (MODIFICADA A DOBLE PRECISION !!!!!!!)
*     - - - - - -     ************************************
*
*  Approximate heliocentric position and velocity of the Earth
*
*  Given:
*     IY       I       year
*     ID       I       day in year (1 = Jan 1st)
*     FD       R       fraction of day
*
*  Returned:
*     PV       R(6)    Earth position & velocity vector
*
*  Notes:
*
*  1  The date and time is TDB (loosely ET) in a Julian calendar
*     which has been aligned to the ordinary Gregorian
*     calendar for the interval 1900 March 1 to 2100 February 28.
*     The year and day can be obtained by calling sla_CALYD or
*     sla_CLYD.
*
*  2  The Earth heliocentric 6-vector is mean equator and equinox
*     of date.  Position part, PV(1-3), is in AU;  velocity part,
*     PV(4-6), is in AU/sec.
*
*  3  Max/RMS errors 1950-2050:
*       13/5 E-5 AU = 19200/7600 km in position
*       47/26 E-10 AU/s = 0.0070/0.0039 km/s in speed
*
*  4  More precise results are obtainable with the routine sla_EVP.
*
*  P.T.Wallace   Starlink   23 November 1994
*
*  Copyright (C) 1995 Rutherford Appleton Laboratory
*-

      IMPLICIT NONE

      INTEGER IY,ID
      double precision FD,PV(6)

      INTEGER IY4
      double precision TWOPI,SPEED,REMB,SEMB,YI,YF,T,ELM,GAMMA,
     :     EM,ELT,EPS0,E,ESQ,V,R,ELMM,COSELT,SINEPS,COSEPS,W1,W2,
     :     SELMM,CELMM

      PARAMETER (TWOPI=6.28318530718d0)

*  Mean orbital speed of Earth, AU/s
      PARAMETER (SPEED=1.9913d-7)

*  Mean Earth:EMB distance and speed, AU and AU/s
      PARAMETER (REMB=3.12d-5,SEMB=8.31d-11)


*  Whole years & fraction of year, and years since J1900.0
      YI=DBLE(IY-1900)
      IY4=MOD(MOD(IY,4)+4,4)
      YF=(DBLE(4*(ID-1/(IY4+1))-IY4-2)+4.0d0*FD)/1461.0d0
      T=YI+YF

*  Geometric mean longitude of Sun
*  (cf 4.881627938d0+6.283319509911d0*T MOD 2PI)
      ELM=MOD(4.881628d0+TWOPI*YF+0.00013420d0*T,TWOPI)

*  Mean longitude of perihelion
      GAMMA=4.908230d0+3.0005d-4*T

*  Mean anomaly
      EM=ELM-GAMMA

*  Mean obliquity
      EPS0=0.40931975d0-2.27d-6*T


*  Eccentricity
      E=0.016751d0-4.2d-7*T
      ESQ=E*E

*  True anomaly
      V=EM+2.0d0*E*SIN(EM)+1.25d0*ESQ*SIN(2.0d0*EM)

*  True ecliptic longitude
      ELT=V+GAMMA

*  True distance
      R=(1.0d0-ESQ)/(1.0d0+E*COS(V))

*  Moon's mean longitude
      ELMM=MOD(4.72d0+83.9971d0*T,TWOPI)

*  Useful functions
      COSELT=COS(ELT)
      SINEPS=SIN(EPS0)
      COSEPS=COS(EPS0)
      W1=-R*SIN(ELT)
      W2=-SPEED*(COSELT+E*COS(GAMMA))
      SELMM=SIN(ELMM)
      CELMM=COS(ELMM)

*  Earth position and velocity
      PV(1)=-R*COSELT-REMB*CELMM
      PV(2)=(W1-REMB*SELMM)*COSEPS
      PV(3)=W1*SINEPS
      PV(4)=SPEED*(SIN(ELT)+E*SIN(GAMMA))+SEMB*SELMM
      PV(5)=(W2-SEMB*CELMM)*COSEPS
      PV(6)=W2*SINEPS

      return
      END
c ------------------------------------------
      SUBROUTINE sla_DMOON (DATE, PV)
*+
*     - - - - - -
*      D M O O N
*     - - - - - -
*
*  Approximate geocentric position and velocity of the Moon
*  (double precision)
*
*  Given:
*     DATE       D       TDB (loosely ET) as a Modified Julian Date
*                                                    (JD-2400000.5)
*
*  Returned:
*     PV         D(6)    Moon x,y,z,xdot,ydot,zdot, mean equator and
*                                         equinox of date (AU, AU/s)
*
*  Notes:
*
*  1  This routine is a full implementation of the algorithm
*     published by Meeus (see reference).
*
*  2  Meeus quotes accuracies of 10 arcsec in longitude, 3 arcsec in
*     latitude and 0.2 arcsec in HP (equivalent to about 20 km in
*     distance).  Comparison with JPL DE200 over the interval
*     1960-2025 gives RMS errors of 3.7 arcsec and 83 mas/hour in
*     longitude, 2.3 arcsec and 48 mas/hour in latitude, 11 km
*     and 81 mm/s in distance.
*
*  3  The original algorithm is expressed in terms of the obsolete
*     timescale Ephemeris Time.  Either TDB or TT can be used, but
*     not UT without incurring significant errors (30 arcsec at
*     the present time) due to the Moon's 0.5 arcsec/sec movement.
*
*  4  The algorithm is based on pre IAU 1976 standards.  However,
*     the result has been moved onto the new (FK5) equinox, an
*     adjustment which is in any case much smaller than the
*     intrinsic accuracy of the procedure.
*
*  5  Velocity is obtained by a complete analytical differentiation
*     of the Meeus model.
*
*  Reference:
*     Meeus, l'Astronomie, June 1984, p348.
*
*  P.T.Wallace   Starlink   7 December 1994
*
*  Copyright (C) 1995 Rutherford Appleton Laboratory
*-

      IMPLICIT NONE

      DOUBLE PRECISION DATE,PV(6)

*  Degrees, arcseconds and seconds of time to radians
      DOUBLE PRECISION D2R,DAS2R,DS2R
      PARAMETER (D2R=0.0174532925199432957692369D0,
     :           DAS2R=4.848136811095359935899141D-6,
     :           DS2R=7.272205216643039903848712D-5)

*  Seconds per Julian century (86400*36525)
      DOUBLE PRECISION CJ
      PARAMETER (CJ=3155760000D0)

*  Julian epoch of B1950
      DOUBLE PRECISION B1950
      PARAMETER (B1950=1949.9997904423D0)

*  Earth equatorial radius in AU ( = 6378.137 / 149597870 )
      DOUBLE PRECISION ERADAU
      PARAMETER (ERADAU=4.2635212653763D-5)

      DOUBLE PRECISION T,THETA,SINOM,COSOM,DOMCOM,WA,DWA,WB,DWB,WOM,
     :                 DWOM,SINWOM,COSWOM,V,DV,COEFF,EMN,EMPN,DN,FN,EN,
     :                 DEN,DTHETA,FTHETA,EL,DEL,B,DB,BF,DBF,P,DP,SP,R,
     :                 DR,X,Y,Z,XD,YD,ZD,SEL,CEL,SB,CB,RCB,RBD,W,EPJ,
     :                 EQCOR,EPS,SINEPS,COSEPS,ES,EC
      INTEGER N,I

*
*  Coefficients for fundamental arguments
*
*   at J1900:  T**0, T**1, T**2, T**3
*   at epoch:  T**0, T**1
*
*  Units are degrees for position and Julian centuries for time
*

*  Moon's mean longitude
      DOUBLE PRECISION ELP0,ELP1,ELP2,ELP3,ELP,DELP
      PARAMETER (ELP0=270.434164D0,
     :           ELP1=481267.8831D0,
     :           ELP2=-0.001133D0,
     :           ELP3=0.0000019D0)

*  Sun's mean anomaly
      DOUBLE PRECISION EM0,EM1,EM2,EM3,EM,DEM
      PARAMETER (EM0=358.475833D0,
     :           EM1=35999.0498D0,
     :           EM2=-0.000150D0,
     :           EM3=-0.0000033D0)

*  Moon's mean anomaly
      DOUBLE PRECISION EMP0,EMP1,EMP2,EMP3,EMP,DEMP

      PARAMETER (EMP0=296.104608D0,
     :           EMP1=477198.8491D0,
     :           EMP2=0.009192D0,
     :           EMP3=0.0000144D0)

*  Moon's mean elongation
      DOUBLE PRECISION D0,D1,D2,D3,D,DD
      PARAMETER (D0=350.737486D0,
     :           D1=445267.1142D0,
     :           D2=-0.001436D0,
     :           D3=0.0000019D0)

*  Mean distance of the Moon from its ascending node
      DOUBLE PRECISION F0,F1,F2,F3,F,DF
      PARAMETER (F0=11.250889D0,
     :           F1=483202.0251D0,
     :           F2=-0.003211D0,
     :           F3=-0.0000003D0)

*  Longitude of the Moon's ascending node
      DOUBLE PRECISION OM0,OM1,OM2,OM3,OM,DOM
      PARAMETER (OM0=259.183275D0,
     :           OM1=-1934.1420D0,
     :           OM2=0.002078D0,
     :           OM3=0.0000022D0)

*  Coefficients for (dimensionless) E factor
      DOUBLE PRECISION E1,E2,E,DE,ESQ,DESQ
      PARAMETER (E1=-0.002495D0,E2=-0.00000752D0)

*  Coefficients for periodic variations etc
      DOUBLE PRECISION PAC,PA0,PA1
      PARAMETER (PAC=0.000233D0,PA0=51.2D0,PA1=20.2D0)
      DOUBLE PRECISION PBC
      PARAMETER (PBC=-0.001778D0)
      DOUBLE PRECISION PCC
      PARAMETER (PCC=0.000817D0)
      DOUBLE PRECISION PDC
      PARAMETER (PDC=0.002011D0)
      DOUBLE PRECISION PEC,PE0,PE1,PE2
      PARAMETER (PEC=0.003964D0,
     :                     PE0=346.560D0,PE1=132.870D0,PE2=-0.0091731D0)
      DOUBLE PRECISION PFC
      PARAMETER (PFC=0.001964D0)
      DOUBLE PRECISION PGC
      PARAMETER (PGC=0.002541D0)
      DOUBLE PRECISION PHC
      PARAMETER (PHC=0.001964D0)
      DOUBLE PRECISION PIC
      PARAMETER (PIC=-0.024691D0)
      DOUBLE PRECISION PJC,PJ0,PJ1
      PARAMETER (PJC=-0.004328D0,PJ0=275.05D0,PJ1=-2.30D0)
      DOUBLE PRECISION CW1
      PARAMETER (CW1=0.0004664D0)
      DOUBLE PRECISION CW2
      PARAMETER (CW2=0.0000754D0)

*
*  Coefficients for Moon position
*
*   Tx(N)       = coefficient of L, B or P term (deg)
*   ITx(N,1-5)  = coefficients of M, M', D, F, E**n in argument
*
      INTEGER NL,NB,NP
      PARAMETER (NL=50,NB=45,NP=31)
      DOUBLE PRECISION TL(NL),TB(NB),TP(NP)
      INTEGER ITL(5,NL),ITB(5,NB),ITP(5,NP)
*
*  Longitude
*                                         M   M'  D   F   n
      DATA TL( 1)/            +6.288750D0                     /,
     :     (ITL(I, 1),I=1,5)/            +0, +1, +0, +0,  0   /
      DATA TL( 2)/            +1.274018D0                     /,
     :     (ITL(I, 2),I=1,5)/            +0, -1, +2, +0,  0   /
      DATA TL( 3)/            +0.658309D0                     /,
     :     (ITL(I, 3),I=1,5)/            +0, +0, +2, +0,  0   /
      DATA TL( 4)/            +0.213616D0                     /,
     :     (ITL(I, 4),I=1,5)/            +0, +2, +0, +0,  0   /
      DATA TL( 5)/            -0.185596D0                     /,
     :     (ITL(I, 5),I=1,5)/            +1, +0, +0, +0,  1   /
      DATA TL( 6)/            -0.114336D0                     /,
     :     (ITL(I, 6),I=1,5)/            +0, +0, +0, +2,  0   /
      DATA TL( 7)/            +0.058793D0                     /,
     :     (ITL(I, 7),I=1,5)/            +0, -2, +2, +0,  0   /
      DATA TL( 8)/            +0.057212D0                     /,
     :     (ITL(I, 8),I=1,5)/            -1, -1, +2, +0,  1   /
      DATA TL( 9)/            +0.053320D0                     /,
     :     (ITL(I, 9),I=1,5)/            +0, +1, +2, +0,  0   /
      DATA TL(10)/            +0.045874D0                     /,
     :     (ITL(I,10),I=1,5)/            -1, +0, +2, +0,  1   /
      DATA TL(11)/            +0.041024D0                     /,
     :     (ITL(I,11),I=1,5)/            -1, +1, +0, +0,  1   /
      DATA TL(12)/            -0.034718D0                     /,
     :     (ITL(I,12),I=1,5)/            +0, +0, +1, +0,  0   /
      DATA TL(13)/            -0.030465D0                     /,
     :     (ITL(I,13),I=1,5)/            +1, +1, +0, +0,  1   /
      DATA TL(14)/            +0.015326D0                     /,
     :     (ITL(I,14),I=1,5)/            +0, +0, +2, -2,  0   /
      DATA TL(15)/            -0.012528D0                     /,
     :     (ITL(I,15),I=1,5)/            +0, +1, +0, +2,  0   /
      DATA TL(16)/            -0.010980D0                     /,
     :     (ITL(I,16),I=1,5)/            +0, -1, +0, +2,  0   /
      DATA TL(17)/            +0.010674D0                     /,
     :     (ITL(I,17),I=1,5)/            +0, -1, +4, +0,  0   /
      DATA TL(18)/            +0.010034D0                     /,
     :     (ITL(I,18),I=1,5)/            +0, +3, +0, +0,  0   /
      DATA TL(19)/            +0.008548D0                     /,
     :     (ITL(I,19),I=1,5)/            +0, -2, +4, +0,  0   /
      DATA TL(20)/            -0.007910D0                     /,
     :     (ITL(I,20),I=1,5)/            +1, -1, +2, +0,  1   /
      DATA TL(21)/            -0.006783D0                     /,
     :     (ITL(I,21),I=1,5)/            +1, +0, +2, +0,  1   /
      DATA TL(22)/            +0.005162D0                     /,
     :     (ITL(I,22),I=1,5)/            +0, +1, -1, +0,  0   /
      DATA TL(23)/            +0.005000D0                     /,
     :     (ITL(I,23),I=1,5)/            +1, +0, +1, +0,  1   /
      DATA TL(24)/            +0.004049D0                     /,
     :     (ITL(I,24),I=1,5)/            -1, +1, +2, +0,  1   /
      DATA TL(25)/            +0.003996D0                     /,
     :     (ITL(I,25),I=1,5)/            +0, +2, +2, +0,  0   /
      DATA TL(26)/            +0.003862D0                     /,
     :     (ITL(I,26),I=1,5)/            +0, +0, +4, +0,  0   /
      DATA TL(27)/            +0.003665D0                     /,
     :     (ITL(I,27),I=1,5)/            +0, -3, +2, +0,  0   /
      DATA TL(28)/            +0.002695D0                     /,
     :     (ITL(I,28),I=1,5)/            -1, +2, +0, +0,  1   /
      DATA TL(29)/            +0.002602D0                     /,
     :     (ITL(I,29),I=1,5)/            +0, +1, -2, -2,  0   /
      DATA TL(30)/            +0.002396D0                     /,
     :     (ITL(I,30),I=1,5)/            -1, -2, +2, +0,  1   /
      DATA TL(31)/            -0.002349D0                     /,
     :     (ITL(I,31),I=1,5)/            +0, +1, +1, +0,  0   /
      DATA TL(32)/            +0.002249D0                     /,
     :     (ITL(I,32),I=1,5)/            -2, +0, +2, +0,  2   /
      DATA TL(33)/            -0.002125D0                     /,
     :     (ITL(I,33),I=1,5)/            +1, +2, +0, +0,  1   /
      DATA TL(34)/            -0.002079D0                     /,
     :     (ITL(I,34),I=1,5)/            +2, +0, +0, +0,  2   /
      DATA TL(35)/            +0.002059D0                     /,
     :     (ITL(I,35),I=1,5)/            -2, -1, +2, +0,  2   /
      DATA TL(36)/            -0.001773D0                     /,
     :     (ITL(I,36),I=1,5)/            +0, +1, +2, -2,  0   /
      DATA TL(37)/            -0.001595D0                     /,
     :     (ITL(I,37),I=1,5)/            +0, +0, +2, +2,  0   /
      DATA TL(38)/            +0.001220D0                     /,
     :     (ITL(I,38),I=1,5)/            -1, -1, +4, +0,  1   /
      DATA TL(39)/            -0.001110D0                     /,
     :     (ITL(I,39),I=1,5)/            +0, +2, +0, +2,  0   /
      DATA TL(40)/            +0.000892D0                     /,
     :     (ITL(I,40),I=1,5)/            +0, +1, -3, +0,  0   /
      DATA TL(41)/            -0.000811D0                     /,
     :     (ITL(I,41),I=1,5)/            +1, +1, +2, +0,  1   /
      DATA TL(42)/            +0.000761D0                     /,
     :     (ITL(I,42),I=1,5)/            -1, -2, +4, +0,  1   /
      DATA TL(43)/            +0.000717D0                     /,
     :     (ITL(I,43),I=1,5)/            -2, +1, +0, +0,  2   /
      DATA TL(44)/            +0.000704D0                     /,
     :     (ITL(I,44),I=1,5)/            -2, +1, -2, +0,  2   /
      DATA TL(45)/            +0.000693D0                     /,
     :     (ITL(I,45),I=1,5)/            +1, -2, +2, +0,  1   /
      DATA TL(46)/            +0.000598D0                     /,
     :     (ITL(I,46),I=1,5)/            -1, +0, +2, -2,  1   /
      DATA TL(47)/            +0.000550D0                     /,
     :     (ITL(I,47),I=1,5)/            +0, +1, +4, +0,  0   /
      DATA TL(48)/            +0.000538D0                     /,
     :     (ITL(I,48),I=1,5)/            +0, +4, +0, +0,  0   /
      DATA TL(49)/            +0.000521D0                     /,
     :     (ITL(I,49),I=1,5)/            -1, +0, +4, +0,  1   /
      DATA TL(50)/            +0.000486D0                     /,
     :     (ITL(I,50),I=1,5)/            +0, +2, -1, +0,  0   /
*
*  Latitude
*                                         M   M'  D   F   n
      DATA TB( 1)/            +5.128189D0                     /,
     :     (ITB(I, 1),I=1,5)/            +0, +0, +0, +1,  0   /
      DATA TB( 2)/            +0.280606D0                     /,
     :     (ITB(I, 2),I=1,5)/            +0, +1, +0, +1,  0   /
      DATA TB( 3)/            +0.277693D0                     /,
     :     (ITB(I, 3),I=1,5)/            +0, +1, +0, -1,  0   /
      DATA TB( 4)/            +0.173238D0                     /,
     :     (ITB(I, 4),I=1,5)/            +0, +0, +2, -1,  0   /
      DATA TB( 5)/            +0.055413D0                     /,
     :     (ITB(I, 5),I=1,5)/            +0, -1, +2, +1,  0   /
      DATA TB( 6)/            +0.046272D0                     /,
     :     (ITB(I, 6),I=1,5)/            +0, -1, +2, -1,  0   /
      DATA TB( 7)/            +0.032573D0                     /,
     :     (ITB(I, 7),I=1,5)/            +0, +0, +2, +1,  0   /
      DATA TB( 8)/            +0.017198D0                     /,
     :     (ITB(I, 8),I=1,5)/            +0, +2, +0, +1,  0   /
      DATA TB( 9)/            +0.009267D0                     /,
     :     (ITB(I, 9),I=1,5)/            +0, +1, +2, -1,  0   /
      DATA TB(10)/            +0.008823D0                     /,
     :     (ITB(I,10),I=1,5)/            +0, +2, +0, -1,  0   /
      DATA TB(11)/            +0.008247D0                     /,
     :     (ITB(I,11),I=1,5)/            -1, +0, +2, -1,  1   /
      DATA TB(12)/            +0.004323D0                     /,
     :     (ITB(I,12),I=1,5)/            +0, -2, +2, -1,  0   /
      DATA TB(13)/            +0.004200D0                     /,
     :     (ITB(I,13),I=1,5)/            +0, +1, +2, +1,  0   /
      DATA TB(14)/            +0.003372D0                     /,
     :     (ITB(I,14),I=1,5)/            -1, +0, -2, +1,  1   /
      DATA TB(15)/            +0.002472D0                     /,
     :     (ITB(I,15),I=1,5)/            -1, -1, +2, +1,  1   /
      DATA TB(16)/            +0.002222D0                     /,
     :     (ITB(I,16),I=1,5)/            -1, +0, +2, +1,  1   /
      DATA TB(17)/            +0.002072D0                     /,
     :     (ITB(I,17),I=1,5)/            -1, -1, +2, -1,  1   /
      DATA TB(18)/            +0.001877D0                     /,
     :     (ITB(I,18),I=1,5)/            -1, +1, +0, +1,  1   /
      DATA TB(19)/            +0.001828D0                     /,
     :     (ITB(I,19),I=1,5)/            +0, -1, +4, -1,  0   /
      DATA TB(20)/            -0.001803D0                     /,
     :     (ITB(I,20),I=1,5)/            +1, +0, +0, +1,  1   /
      DATA TB(21)/            -0.001750D0                     /,
     :     (ITB(I,21),I=1,5)/            +0, +0, +0, +3,  0   /
      DATA TB(22)/            +0.001570D0                     /,
     :     (ITB(I,22),I=1,5)/            -1, +1, +0, -1,  1   /
      DATA TB(23)/            -0.001487D0                     /,
     :     (ITB(I,23),I=1,5)/            +0, +0, +1, +1,  0   /
      DATA TB(24)/            -0.001481D0                     /,
     :     (ITB(I,24),I=1,5)/            +1, +1, +0, +1,  1   /
      DATA TB(25)/            +0.001417D0                     /,
     :     (ITB(I,25),I=1,5)/            -1, -1, +0, +1,  1   /
      DATA TB(26)/            +0.001350D0                     /,
     :     (ITB(I,26),I=1,5)/            -1, +0, +0, +1,  1   /
      DATA TB(27)/            +0.001330D0                     /,
     :     (ITB(I,27),I=1,5)/            +0, +0, -1, +1,  0   /
      DATA TB(28)/            +0.001106D0                     /,
     :     (ITB(I,28),I=1,5)/            +0, +3, +0, +1,  0   /
      DATA TB(29)/            +0.001020D0                     /,
     :     (ITB(I,29),I=1,5)/            +0, +0, +4, -1,  0   /
      DATA TB(30)/            +0.000833D0                     /,
     :     (ITB(I,30),I=1,5)/            +0, -1, +4, +1,  0   /
      DATA TB(31)/            +0.000781D0                     /,
     :     (ITB(I,31),I=1,5)/            +0, +1, +0, -3,  0   /
      DATA TB(32)/            +0.000670D0                     /,
     :     (ITB(I,32),I=1,5)/            +0, -2, +4, +1,  0   /
      DATA TB(33)/            +0.000606D0                     /,
     :     (ITB(I,33),I=1,5)/            +0, +0, +2, -3,  0   /
      DATA TB(34)/            +0.000597D0                     /,
     :     (ITB(I,34),I=1,5)/            +0, +2, +2, -1,  0   /
      DATA TB(35)/            +0.000492D0                     /,
     :     (ITB(I,35),I=1,5)/            -1, +1, +2, -1,  1   /
      DATA TB(36)/            +0.000450D0                     /,
     :     (ITB(I,36),I=1,5)/            +0, +2, -2, -1,  0   /
      DATA TB(37)/            +0.000439D0                     /,
     :     (ITB(I,37),I=1,5)/            +0, +3, +0, -1,  0   /
      DATA TB(38)/            +0.000423D0                     /,
     :     (ITB(I,38),I=1,5)/            +0, +2, +2, +1,  0   /
      DATA TB(39)/            +0.000422D0                     /,
     :     (ITB(I,39),I=1,5)/            +0, -3, +2, -1,  0   /
      DATA TB(40)/            -0.000367D0                     /,
     :     (ITB(I,40),I=1,5)/            +1, -1, +2, +1,  1   /
      DATA TB(41)/            -0.000353D0                     /,
     :     (ITB(I,41),I=1,5)/            +1, +0, +2, +1,  1   /
      DATA TB(42)/            +0.000331D0                     /,
     :     (ITB(I,42),I=1,5)/            +0, +0, +4, +1,  0   /
      DATA TB(43)/            +0.000317D0                     /,
     :     (ITB(I,43),I=1,5)/            -1, +1, +2, +1,  1   /
      DATA TB(44)/            +0.000306D0                     /,
     :     (ITB(I,44),I=1,5)/            -2, +0, +2, -1,  2   /
      DATA TB(45)/            -0.000283D0                     /,
     :     (ITB(I,45),I=1,5)/            +0, +1, +0, +3,  0   /
*
*  Parallax
*                                         M   M'  D   F   n
      DATA TP( 1)/            +0.950724D0                     /,
     :     (ITP(I, 1),I=1,5)/            +0, +0, +0, +0,  0   /
      DATA TP( 2)/            +0.051818D0                     /,
     :     (ITP(I, 2),I=1,5)/            +0, +1, +0, +0,  0   /
      DATA TP( 3)/            +0.009531D0                     /,
     :     (ITP(I, 3),I=1,5)/            +0, -1, +2, +0,  0   /
      DATA TP( 4)/            +0.007843D0                     /,
     :     (ITP(I, 4),I=1,5)/            +0, +0, +2, +0,  0   /
      DATA TP( 5)/            +0.002824D0                     /,
     :     (ITP(I, 5),I=1,5)/            +0, +2, +0, +0,  0   /
      DATA TP( 6)/            +0.000857D0                     /,
     :     (ITP(I, 6),I=1,5)/            +0, +1, +2, +0,  0   /
      DATA TP( 7)/            +0.000533D0                     /,
     :     (ITP(I, 7),I=1,5)/            -1, +0, +2, +0,  1   /
      DATA TP( 8)/            +0.000401D0                     /,
     :     (ITP(I, 8),I=1,5)/            -1, -1, +2, +0,  1   /
      DATA TP( 9)/            +0.000320D0                     /,
     :     (ITP(I, 9),I=1,5)/            -1, +1, +0, +0,  1   /
      DATA TP(10)/            -0.000271D0                     /,
     :     (ITP(I,10),I=1,5)/            +0, +0, +1, +0,  0   /
      DATA TP(11)/            -0.000264D0                     /,
     :     (ITP(I,11),I=1,5)/            +1, +1, +0, +0,  1   /
      DATA TP(12)/            -0.000198D0                     /,
     :     (ITP(I,12),I=1,5)/            +0, -1, +0, +2,  0   /
      DATA TP(13)/            +0.000173D0                     /,
     :     (ITP(I,13),I=1,5)/            +0, +3, +0, +0,  0   /
      DATA TP(14)/            +0.000167D0                     /,
     :     (ITP(I,14),I=1,5)/            +0, -1, +4, +0,  0   /
      DATA TP(15)/            -0.000111D0                     /,
     :     (ITP(I,15),I=1,5)/            +1, +0, +0, +0,  1   /
      DATA TP(16)/            +0.000103D0                     /,
     :     (ITP(I,16),I=1,5)/            +0, -2, +4, +0,  0   /
      DATA TP(17)/            -0.000084D0                     /,
     :     (ITP(I,17),I=1,5)/            +0, +2, -2, +0,  0   /
      DATA TP(18)/            -0.000083D0                     /,
     :     (ITP(I,18),I=1,5)/            +1, +0, +2, +0,  1   /
      DATA TP(19)/            +0.000079D0                     /,
     :     (ITP(I,19),I=1,5)/            +0, +2, +2, +0,  0   /
      DATA TP(20)/            +0.000072D0                     /,
     :     (ITP(I,20),I=1,5)/            +0, +0, +4, +0,  0   /
      DATA TP(21)/            +0.000064D0                     /,
     :     (ITP(I,21),I=1,5)/            -1, +1, +2, +0,  1   /
      DATA TP(22)/            -0.000063D0                     /,
     :     (ITP(I,22),I=1,5)/            +1, -1, +2, +0,  1   /
      DATA TP(23)/            +0.000041D0                     /,
     :     (ITP(I,23),I=1,5)/            +1, +0, +1, +0,  1   /
      DATA TP(24)/            +0.000035D0                     /,
     :     (ITP(I,24),I=1,5)/            -1, +2, +0, +0,  1   /
      DATA TP(25)/            -0.000033D0                     /,
     :     (ITP(I,25),I=1,5)/            +0, +3, -2, +0,  0   /
      DATA TP(26)/            -0.000030D0                     /,
     :     (ITP(I,26),I=1,5)/            +0, +1, +1, +0,  0   /
      DATA TP(27)/            -0.000029D0                     /,
     :     (ITP(I,27),I=1,5)/            +0, +0, -2, +2,  0   /
      DATA TP(28)/            -0.000029D0                     /,
     :     (ITP(I,28),I=1,5)/            +1, +2, +0, +0,  1   /
      DATA TP(29)/            +0.000026D0                     /,
     :     (ITP(I,29),I=1,5)/            -2, +0, +2, +0,  2   /
      DATA TP(30)/            -0.000023D0                     /,
     :     (ITP(I,30),I=1,5)/            +0, +1, -2, +2,  0   /
      DATA TP(31)/            +0.000019D0                     /,
     :     (ITP(I,31),I=1,5)/            -1, -1, +4, +0,  1   /



*  Centuries since J1900
      T=(DATE-15019.5D0)/36525D0

*
*  Fundamental arguments (radians) and derivatives (radians per
*  Julian century) for the current epoch
*

*  Moon's mean longitude
      ELP=D2R*MOD(ELP0+(ELP1+(ELP2+ELP3*T)*T)*T,360D0)
      DELP=D2R*(ELP1+(2D0*ELP2+3D0*ELP3*T)*T)

*  Sun's mean anomaly
      EM=D2R*MOD(EM0+(EM1+(EM2+EM3*T)*T)*T,360D0)
      DEM=D2R*(EM1+(2D0*EM2+3D0*EM3*T)*T)

*  Moon's mean anomaly
      EMP=D2R*MOD(EMP0+(EMP1+(EMP2+EMP3*T)*T)*T,360D0)
      DEMP=D2R*(EMP1+(2D0*EMP2+3D0*EMP3*T)*T)

*  Moon's mean elongation
      D=D2R*MOD(D0+(D1+(D2+D3*T)*T)*T,360D0)
      DD=D2R*(D1+(2D0*D2+3D0*D3*T)*T)

*  Mean distance of the Moon from its ascending node
      F=D2R*MOD(F0+(F1+(F2+F3*T)*T)*T,360D0)
      DF=D2R*(F1+(2D0*F2+3D0*F3*T)*T)

*  Longitude of the Moon's ascending node
      OM=D2R*MOD(OM0+(OM1+(OM2+OM3*T)*T)*T,360D0)
      DOM=D2R*(OM1+(2D0*OM2+3D0*OM3*T)*T)
      SINOM=SIN(OM)
      COSOM=COS(OM)
      DOMCOM=DOM*COSOM
*  Add the periodic variations
      THETA=D2R*(PA0+PA1*T)
      WA=SIN(THETA)
      DWA=D2R*PA1*COS(THETA)
      THETA=D2R*(PE0+(PE1+PE2*T)*T)
      WB=PEC*SIN(THETA)
      DWB=D2R*PEC*(PE1+2D0*PE2*T)*COS(THETA)
      ELP=ELP+D2R*(PAC*WA+WB+PFC*SINOM)
      DELP=DELP+D2R*(PAC*DWA+DWB+PFC*DOMCOM)
      EM=EM+D2R*PBC*WA
      DEM=DEM+D2R*PBC*DWA
      EMP=EMP+D2R*(PCC*WA+WB+PGC*SINOM)
      DEMP=DEMP+D2R*(PCC*DWA+DWB+PGC*DOMCOM)
      D=D+D2R*(PDC*WA+WB+PHC*SINOM)
      DD=DD+D2R*(PDC*DWA+DWB+PHC*DOMCOM)
      WOM=OM+D2R*(PJ0+PJ1*T)
      DWOM=DOM+D2R*PJ1
      SINWOM=SIN(WOM)
      COSWOM=COS(WOM)
      F=F+D2R*(WB+PIC*SINOM+PJC*SINWOM)
      DF=DF+D2R*(DWB+PIC*DOMCOM+PJC*DWOM*COSWOM)

*  E-factor, and square
      E=1D0+(E1+E2*T)*T
      DE=E1+2D0*E2*T
      ESQ=E*E
      DESQ=2D0*E*DE

*
*  Series expansions
*

*  Longitude
      V=0D0
      DV=0D0
      DO N=NL,1,-1
         COEFF=TL(N)
         EMN=DBLE(ITL(1,N))
         EMPN=DBLE(ITL(2,N))
         DN=DBLE(ITL(3,N))
         FN=DBLE(ITL(4,N))
         I=ITL(5,N)
         IF (I.EQ.0) THEN
            EN=1D0
            DEN=0D0
         ELSE IF (I.EQ.1) THEN
            EN=E
            DEN=DE
         ELSE
            EN=ESQ
            DEN=DESQ
         END IF
         THETA=EMN*EM+EMPN*EMP+DN*D+FN*F
         DTHETA=EMN*DEM+EMPN*DEMP+DN*DD+FN*DF
         FTHETA=SIN(THETA)
         V=V+COEFF*FTHETA*EN
         DV=DV+COEFF*(COS(THETA)*DTHETA*EN+FTHETA*DEN)
      END DO
      EL=ELP+D2R*V
      DEL=(DELP+D2R*DV)/CJ

*  Latitude
      V=0D0
      DV=0D0
      DO N=NB,1,-1
         COEFF=TB(N)
         EMN=DBLE(ITB(1,N))
         EMPN=DBLE(ITB(2,N))
         DN=DBLE(ITB(3,N))
         FN=DBLE(ITB(4,N))
         I=ITB(5,N)
         IF (I.EQ.0) THEN
            EN=1D0
            DEN=0D0
         ELSE IF (I.EQ.1) THEN
            EN=E
            DEN=DE
         ELSE
            EN=ESQ
            DEN=DESQ
         END IF
         THETA=EMN*EM+EMPN*EMP+DN*D+FN*F
         DTHETA=EMN*DEM+EMPN*DEMP+DN*DD+FN*DF
         FTHETA=SIN(THETA)
         V=V+COEFF*FTHETA*EN
         DV=DV+COEFF*(COS(THETA)*DTHETA*EN+FTHETA*DEN)
      END DO
      BF=1D0-CW1*COSOM-CW2*COSWOM
      DBF=CW1*DOM*SINOM+CW2*DWOM*SINWOM
      B=D2R*V*BF
      DB=D2R*(DV*BF+V*DBF)/CJ

*  Parallax
      V=0D0
      DV=0D0
      DO N=NP,1,-1
         COEFF=TP(N)
         EMN=DBLE(ITP(1,N))
         EMPN=DBLE(ITP(2,N))
         DN=DBLE(ITP(3,N))
         FN=DBLE(ITP(4,N))
         I=ITP(5,N)
         IF (I.EQ.0) THEN
            EN=1D0
            DEN=0D0
         ELSE IF (I.EQ.1) THEN
            EN=E
            DEN=DE
         ELSE
            EN=ESQ
            DEN=DESQ
         END IF
         THETA=EMN*EM+EMPN*EMP+DN*D+FN*F
         DTHETA=EMN*DEM+EMPN*DEMP+DN*DD+FN*DF
         FTHETA=COS(THETA)
         V=V+COEFF*FTHETA*EN
         DV=DV+COEFF*(-SIN(THETA)*DTHETA*EN+FTHETA*DEN)
      END DO
      P=D2R*V
      DP=D2R*DV/CJ

*
*  Transformation into final form
*

*  Parallax to distance (AU, AU/sec)
      SP=SIN(P)
      R=ERADAU/SP
      DR=-R*DP*COS(P)/SP

*  Longitude, latitude to x,y,z (AU)
      SEL=SIN(EL)
      CEL=COS(EL)
      SB=SIN(B)
      CB=COS(B)
      RCB=R*CB
      RBD=R*DB
      W=RBD*SB-CB*DR
      X=RCB*CEL
      Y=RCB*SEL
      Z=R*SB
      XD=-Y*DEL-W*CEL
      YD=X*DEL-W*SEL
      ZD=RBD*CB+SB*DR

*  Julian centuries since J2000
      T=(DATE-51544.5D0)/36525D0

*  Fricke equinox correction
      EPJ=2000D0+T*100D0
      EQCOR=DS2R*(0.035D0+0.00085D0*(EPJ-B1950))

*  Mean obliquity (IAU 1976)
      EPS=DAS2R*(84381.448D0+(-46.8150D0+(-0.00059D0+0.001813D0*T)*T)*T)

*  To the equatorial system, mean of date, FK5 system
      SINEPS=SIN(EPS)
      COSEPS=COS(EPS)
      ES=EQCOR*SINEPS
      EC=EQCOR*COSEPS
      PV(1)=X-EC*Y+ES*Z
      PV(2)=EQCOR*X+Y*COSEPS-Z*SINEPS
      PV(3)=Y*SINEPS+Z*COSEPS
      PV(4)=XD-EC*YD+ES*ZD
      PV(5)=EQCOR*XD+YD*COSEPS-ZD*SINEPS
      PV(6)=YD*SINEPS+ZD*COSEPS

      return
      END

c __________________________________________________________________


        subroutine rot1(ang,r,rp)
        implicit double precision (a-h,o-z)
        dimension r(3),rp(3)


        rp(1)=r(1)
        rp(2)=dcos(ang)*r(2)+dsin(ang)*r(3)
        rp(3)=-dsin(ang)*r(2)+dcos(ang)*r(3)

        return
        end

c __________________________________________________________________


        subroutine rot2(ang,r,rp)
        implicit double precision (a-h,o-z)
        dimension r(3),rp(3)


        rp(1)=dcos(ang)*r(1)-dsin(ang)*r(3)
        rp(2)=r(2)
        rp(3)=dsin(ang)*r(1)+dcos(ang)*r(3)

        return
        end




c __________________________________________________________________



        subroutine rot3(ang,r,rp)
        implicit double precision (a-h,o-z)
        dimension r(3),rp(3)


        rp(1)=dcos(ang)*r(1)+dsin(ang)*r(2)
        rp(2)=-dsin(ang)*r(1)+dcos(ang)*r(2)
        rp(3)=r(3)

        return
        end
c ------------------------------------------------------------------

c -------------------------------------------------------------------
        subroutine yydoy2MJD(iy,idoy,fr,xmjd)
        implicit double precision (a-h,o-z)

c       --------------------------------------------------------------
c       [year, day_of_year, fraction_of_day] ---> Modified Julian Date
c                                                       (JD-2400000.5)
c       --------------------------------------------------------------



c   ....Julian date .......................................

        xy=dble(iy)-1.d0
        xm=1.d0+12.d0
        xd=dble(idoy)
        xt=fr*24.d0

        xjd=int(365.25d0*xy)+int(30.6001d0*(xm+1.d0))+
     *      xd+xt/24.d0+1720981.5d0

c   ........................................................

c   ....Modifeid Julian Date (JD-2400000.5) ................
        xMJD=xjd-2400000.5d0
c       print *, "A",xMJD
c   ........................................................


        return
        end


c ------------------------------------------------------------------


        subroutine yyddUT2sid(iy,idoy,UT,sid)
        implicit double precision (a-h,o-z)
        double precision jd

c       ----------------------------------------------------
c       anyo dia_del_anyo UT -->|yyddUT2sid|--> sid(hh.hh)
c       ----------------------------------------------------


        xy=dble(iy)-1.d0
        xm=1.d0+12.d0
        xd=dble(idoy)

c       ....dia juliano (jd).........................

        jd=int(365.25d0*xy)+int(30.6001d0*(xm+1.d0))+
     *      xd+UT/24.d0+1720981.5d0
c       .............................................

c -----------------------------------------------------------
        sec=(jd-2451545.d0)*24.d0*3600.d0


c        print *, "AA",jd,sec
c   ....................................................
c -----------------------------------------------------------
c        print *, "AA",jd,sec
c   ....................................................
c -----------------------------------------------------------

c       ....Calculo tiempo sidereo ...........................
        tt=(jd-2451545.d0)/36525.d0

        sid=24110.54841d0+8640184.812866d0*tt+0.093104d0*tt**2
     * -(6.2d-6)*(tt**3)
        sid=(sid/3600.d0+UT)
        sid=mod(sid,24.d0)
        if (sid.lt.0.d0) sid=sid+24.d0
c       ......................................................


c       print *, "BB",sid

        return
        end
c ------------------------------------------------------------------

       SUBROUTINE TIDE(XSTA,iYR,iMONTH,iDAY,FHR,XSUN,XMON,DXTIDE)
C  
C PURPOSE    :  COMPUTATION OF TIDAL CORRECTIONS OF STATION DISPLACEMENTS  
C               CAUSED BY LUNAR AND SOLAR GRAVITATIONAL ATTRACTION  
C               (SEE IERS STANDARDS 1996)  
C         STEP 1 (HERE GENERAL DEGREE 2 AND 3 CORRECTIONS +  
C                CALL ST1IDIU + CALL ST1ISEM + CALL ST1L1) 
C         + STEP 2 (CALL STEP2DIU + CALL STEP2LON + CALL STEP2IDIU)  
C IT HAS BEEN DECIDED THAT THE STEP 3 NON-CORRECTION FOR PERMANENT TIDE 
C WOULD NOT BE APPLIED IN ORDER TO AVOID JUMP IN THE REFERENCE FRAME 
C (THIS STEP 3 MUST ADDED IN ORDER TO GET THE NON-TIDAL STATION POSITION  
C AND TO BE CONFORMED WITH THE IAG RESOLUTION.)  
C  
C    INPUT :  XSTA(I),I=1,2,3: GEOCENTRIC POSITION OF THE STATION (ITRF,
C                              CO-ROTATING FRAME) -- UNITS = METERS
C             XSUN(I),I=1,2,3: GEOC. POSITION OF THE SUN (ECEF FRAME) --
C                              UNITS = METERS
C             XMON(I),I=1,2,3: GEOC. POSITION OF THE MOON (ECEF FRAME) --
C                              UNITS = METERS
C             IYR : YEAR (UTC TIMESCALE)
C             IMONTH : MONTH (UTC TIMESCALE)
C             IDAY : DAY (UTC TIMESCALE)
C             FHR=hr+zmin/60.+sec/3600. : HR IN THE DAY 
C   OUTPUT :  DXTIDE(I),I=1,2,3: DISPLACEMENT VECTOR (GEOCENTRIC ITRF FRAME) --
C                                UNITS = METERS
C  
C SUBROUTINES CALLED  :  SPROD  
C                        ST1IDIU 
C                        ST1ISEM 
C                        ST1L1 
C                        STEP2DIU  
C                        STEP2LON 
C                        STEP2ILON  
C  
C AUTHOR IERS 1996 :  V. DEHANT, S. MATHEWS AND J. GIPSON
C    (TEST BETWEEN TWO SUBROUTINES) 
C AUTHOR IERS 2000 :  V. DEHANT AND S. MATHEWS
C    (TEST IN THE BERNESE PROGRAM BY C. BRUYNINX)
C  
C CREATED    :  96/03/23              LAST MODIFIED :  00/05/17 14:10  
C UPDATED    :2006/02/06  HEADER COMMENTS MODIFIED TO CLARIFY INPUT/OUTPUT
C                         UNITS AND SYSTEMS BY JIM RAY
C UPDATED    :2006/02/06  SUBROUTINE DUTC MODIFIED FOR LEAP SECOND ON
C                         2006.0 AND TO CORRECT do 5 i=1,87 from 84 to 87
C                         BY JIM RAY
C UPDATED    :2006/08/31  CORRECT DUTC FOR DATES AFTER 2007 (G. PETIT)
C UPDATED    :2007/06/20  MODIFIED SUBROUTINE DUTC TO CORRECT PAST MISTAKE (PROVIDED BY H. MANCHE) 
C                         CORRECT STEP2DIU LINE       DE=DATDI(8,J)*SINPHI*COS(THETAF+ZLA)+
C                                            TO       DE=DATDI(8,J)*SINPHI*COS(THETAF+ZLA)-
C UPDATED    :2007/10/23  Replace subroutines DUTC and FJLDY by SOFA routines iau_CAL2JD and iau_DAT.
C                         The source code of these routines is included in this file. 
C                         Correct time arguments of subroutine STEP2DIU: argument T is in TT, FHR is in UT.
C                         By H. Manche and G. Petit
C  
C  
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C  
      DOUBLE PRECISION XSTA(3),XSUN(3),XMON(3),DXTIDE(3),XCORSTA(3)
      DOUBLE PRECISION H20,L20,H3,L3,H2,L2
      DOUBLE PRECISION mass_ratio_sun,mass_ratio_moon
      integer statut
      DOUBLE PRECISION JJM0,JJM1
C  
C NOMINAL SECOND DEGREE AND THIRD DEGREE LOVE NUMBERS AND SHIDA NUMBERS  
C ---------------------------------------------------------------------  
      DATA H20/0.6078D0/,L20/0.0847D0/,H3/0.292D0/,L3/0.015D0/
C  
C SCALAR PRODUCT OF STATION VECTOR WITH SUN/MOON VECTOR  
C -----------------------------------------------------  
      CALL SPROD(XSTA,XSUN,SCS,RSTA,RSUN)  
      CALL SPROD(XSTA,XMON,SCM,RSTA,RMON)  
      SCSUN=SCS/RSTA/RSUN  
      SCMON=SCM/RSTA/RMON
C   
C COMPUTATION OF NEW H2 AND L2  
C ----------------------------  
      COSPHI=DSQRT(XSTA(1)**2+XSTA(2)**2)/RSTA
      H2=H20-0.0006*(1.-3./2.*COSPHI**2)
      L2=L20+0.0002*(1.-3./2.*COSPHI**2)  
C
C P2-TERM  
C -------  
      P2SUN=3.*(H2/2.-L2)*SCSUN**2-H2/2.  
      P2MON=3.*(H2/2.-L2)*SCMON**2-H2/2.  
C  
C P3-TERM  
C -------  
      P3SUN=5./2.*(H3-3.*L3)*SCSUN**3+3./2.*(L3-H3)*SCSUN
      P3MON=5./2.*(H3-3.*L3)*SCMON**3+3./2.*(L3-H3)*SCMON

C  
C TERM IN DIRECTION OF SUN/MOON VECTOR  
C ------------------------------------  
      X2SUN=3.*L2*SCSUN  
      X2MON=3.*L2*SCMON  
      X3SUN=3.*L3/2.*(5.*SCSUN**2-1.)  
      X3MON=3.*L3/2.*(5.*SCMON**2-1.)
C  
C FACTORS FOR SUN/MOON  
C --------------------  
      MASS_RATIO_SUN=332945.943062d0
      MASS_RATIO_MOON=0.012300034d0
      RE =6378136.55d0
      FAC2SUN=MASS_RATIO_SUN*RE*(RE/RSUN)**3
      FAC2MON=MASS_RATIO_MOON*RE*(RE/RMON)**3
      FAC3SUN=FAC2SUN*(RE/RSUN)
      FAC3MON=FAC2MON*(RE/RMON)
C  
C TOTAL DISPLACEMENT  
C ------------------  
      DO 10 I=1,3  
      DXTIDE(I)=FAC2SUN*( X2SUN*XSUN(I)/RSUN + P2SUN*XSTA(I)/RSTA ) +
     1            FAC2MON*( X2MON*XMON(I)/RMON + P2MON*XSTA(I)/RSTA ) +  
     2            FAC3SUN*( X3SUN*XSUN(I)/RSUN + P3SUN*XSTA(I)/RSTA ) +   
     3            FAC3MON*( X3MON*XMON(I)/RMON + P3MON*XSTA(I)/RSTA )  
10    CONTINUE  
      call zero_vec8(xcorsta)
C  
C CORRECTIONS FOR THE OUT-OF-PHASE PART OF LOVE NUMBERS (PART H_2^(0)I  
C            AND L_2^(0)I )  
C FIRST, FOR THE DIURNAL BAND       

      CALL ST1IDIU(XSTA,XSUN,XMON,FAC2SUN,FAC2MON,XCORSTA)
      DO 11 I=1,3
      DXTIDE(I)=DXTIDE(I)+XCORSTA(I)  
11    CONTINUE
C  
C SECOND, FOR THE SEMI-DIURNAL BAND       
C 
      CALL ST1ISEM(XSTA,XSUN,XMON,FAC2SUN,FAC2MON,XCORSTA)
      DO 12 I=1,3
      DXTIDE(I)=DXTIDE(I)+XCORSTA(I)
12    CONTINUE  
C  
C CORRECTIONS FOR THE LATITUDE DEPENDENCE OF LOVE NUMBERS (PART L^(1) )  
C   
      CALL ST1L1(XSTA,XSUN,XMON,FAC2SUN,FAC2MON,XCORSTA)
      DO 13 I=1,3  
      DXTIDE(I)=DXTIDE(I)+XCORSTA(I)
13    CONTINUE    
C  
C CONSIDER CORRECTIONS FOR STEP 2  
C  
C CORRECTIONS FOR THE DIURNAL BAND:  
C 
C  FIRST, WE NEED TO KNOW THE DATE CONVERTED IN JULIAN CENTURIES 
c        
c   1) CALL THE SUBROUTINE COMPUTING THE JULIAN DATE 
c 
      call iau_CAL2JD ( iyr, imonth, iday, JJM0, JJM1, statut )
      T=((JJM0-2451545.0d0)+JJM1+FHR/24.0d0)/36525.d0
c 
c   2) CALL THE SUBROUTINE COMPUTING THE CORRECTION OF UTC TIME  
c 
      call iau_DAT ( IYR,IMONTH,IDAY, FHR, DTT, statut )
      DTT = DTT + 32.184d0
c     Conversion of T in TT time
      T=T+DTT/(3600.0d0*24.0d0*36525.0d0)
C  
C  SECOND, WE CAN CALL THE SUBROUTINE STEP2DIU, FOR THE DIURNAL BAND CORRECTIONS,
C   (in-phase and out-of-phase frequency dependence):
C  
      CALL STEP2DIU(XSTA,FHR,T,XCORSTA)  
      DO 14 I=1,3  
      DXTIDE(I)=DXTIDE(I)+XCORSTA(I)
14    CONTINUE  
C  
C  CORRECTIONS FOR THE LONG-PERIOD BAND,
C   (in-phase and out-of-phase frequency dependence):  
C
      call STEP2LON(XSTA,FHR,T,XCORSTA)
      DO 15 I=1,3  
      DXTIDE(I)=DXTIDE(I)+XCORSTA(I)
15    CONTINUE  
C    
C CONSIDER CORRECTIONS FOR STEP 3  
C  
C UNCORRECT FOR THE PERMANENT TIDE  
C  
C      PI=3.141592654
C      SINPHI=XSTA(3)/RSTA  
C      COSPHI=dsqrt(XSTA(1)**2+XSTA(2)**2)/RSTA
C      COSLA=XSTA(1)/COSPHI/RSTA  
C      SINLA=XSTA(2)/COSPHI/RSTA  
C      DR=-DSQRT(5./4./PI)*H2*0.31460*(3./2.*SINPHI**2-0.5)
C      DN=-DSQRT(5./4./PI)*L2*0.31460*3.*COSPHI*SINPHI
C      DXTIDE(1)=DXTIDE(1)-DR*COSLA*COSPHI+DN*COSLA*SINPHI
C      DXTIDE(2)=DXTIDE(2)-DR*SINLA*COSPHI+DN*SINLA*SINPHI  
C      DXTIDE(3)=DXTIDE(3)-DR*SINPHI      -DN*COSPHI
C       
      RETURN  
      END  
C********************************************************************
C 
C 
      SUBROUTINE SPROD(X,Y,SCAL,R1,R2) 
C 
C  COMPUTATION OF THE SCALAR-PRODUCT OF TWO VECTORS AND THEIR NORMS 
C 
C  INPUT :  X(I),I=1,2,3: COMPONENTS OF VECTOR X 
C           Y(I),I=1,2,3: COMPONENTS OF VECTOR Y 
C  OUTPUT :  SCAL: SCALAR PRODUCT OF X AND Y 
C            R1,R2  : LENGTHS OF THE TWO VECTORS X AND Y 
C 
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DOUBLE PRECISION X(3),Y(3)
      R1=DSQRT(X(1)**2+X(2)**2+X(3)**2) 
      R2=DSQRT(Y(1)**2+Y(2)**2+Y(3)**2) 
      SCAL=X(1)*Y(1)+X(2)*Y(2)+X(3)*Y(3) 
      RETURN 
      END 
C 
C-------------------------------------------------------------------------
C
      SUBROUTINE ST1L1(XSTA,XSUN,XMON,FAC2SUN,FAC2MON,XCORSTA)
C  
C THIS SUBROUTINE GIVES THE CORRECTIONS INDUCED BY THE LATITUDE DEPENDENCE  
C GIVEN BY L^(1) IN MAHTEWS ET AL (1991)  
C  
C       INPUT : XSTA,XSUN,XMON,FAC3SUN,FAC3MON  
C      OUTPUT : XCORSTA  
C  
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C compute the norm of a vector
      DOUBLE PRECISION norm8
      DIMENSION XSTA(3),XSUN(3),XMON(3),XCORSTA(3)  
      DOUBLE PRECISION L1,L1D,L1SD
      DATA L1D/0.0012d0/,L1SD/0.0024d0/
      RSTA=norm8(xsta)
      SINPHI=XSTA(3)/RSTA  
      COSPHI=DSQRT(XSTA(1)**2+XSTA(2)**2)/RSTA  
      SINLA=XSTA(2)/COSPHI/RSTA  
      COSLA=XSTA(1)/COSPHI/RSTA  
      RMON=norm8(XMON)
      rsun=norm8(xsun)
C
C FOR THE DIURNAL BAND  
C  
      L1=L1D  
      DNSUN=-L1*SINPHI**2*FAC2SUN*XSUN(3)*(XSUN(1)*COSLA+XSUN(2)*SINLA)
     &            /RSUN**2
      DNMON=-L1*SINPHI**2*FAC2MON*XMON(3)*(XMON(1)*COSLA+XMON(2)*SINLA)
     &            /RMON**2
      DESUN=L1*SINPHI*(COSPHI**2-SINPHI**2)*FAC2SUN*XSUN(3)*
     1 (XSUN(1)*SINLA-XSUN(2)*COSLA)/RSUN**2
      DEMON=L1*SINPHI*(COSPHI**2-SINPHI**2)*FAC2MON*XMON(3)*
     1 (XMON(1)*SINLA-XMON(2)*COSLA)/RMON**2
      DE=3.*(DESUN+DEMON)  
      DN=3.*(DNSUN+DNMON)  
      XCORSTA(1)=-DE*SINLA-DN*SINPHI*COSLA  
      XCORSTA(2)=DE*COSLA-DN*SINPHI*SINLA  
      XCORSTA(3)=DN*COSPHI  
C   
C FOR THE SEMI-DIURNAL BAND  
C  
      L1=L1SD  
      COSTWOLA=COSLA**2-SINLA**2  
      SINTWOLA=2.*COSLA*SINLA  
      DNSUN=-L1/2.*SINPHI*COSPHI*FAC2SUN*((XSUN(1)**2-XSUN(2)**2)*
     1 COSTWOLA+2.*XSUN(1)*XSUN(2)*SINTWOLA)/RSUN**2
      DNMON=-L1/2.*SINPHI*COSPHI*FAC2MON*((XMON(1)**2-XMON(2)**2)*
     1 COSTWOLA+2.*XMON(1)*XMON(2)*SINTWOLA)/RMON**2
      DESUN=-L1/2.*SINPHI**2*COSPHI*FAC2SUN*((XSUN(1)**2-XSUN(2)**2)*
     1 SINTWOLA-2.*XSUN(1)*XSUN(2)*COSTWOLA)/RSUN**2
      DEMON=-L1/2.*SINPHI**2*COSPHI*FAC2MON*((XMON(1)**2-XMON(2)**2)*
     1 SINTWOLA-2.*XMON(1)*XMON(2)*COSTWOLA)/RMON**2
      DE=3.*(DESUN+DEMON)  
      DN=3.*(DNSUN+DNMON)  
      XCORSTA(1)=XCORSTA(1)-DE*SINLA-DN*SINPHI*COSLA  
      XCORSTA(2)=XCORSTA(2)+DE*COSLA-DN*SINPHI*SINLA  
      XCORSTA(3)=XCORSTA(3)+DN*COSPHI  
      RETURN  
      END  

C*************************************************************************
C     Last change:  VD   17 May 00   1:20 pm
C  THESE ARE THE SUBROUTINES FOR THE STEP2 OF THE TIDAL CORRECTIONS. 
C  THEY ARE CALLED TO ACCOUNT FOR THE FREQUENCY DEPENDENCE  
C  OF THE LOVE NUMBERS. 
C 
      SUBROUTINE STEP2DIU(XSTA,FHR,T,XCORSTA)  
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DOUBLE PRECISION XSTA(3),XCORSTA(3),DATDI(9,31)
      DOUBLE PRECISION deg2rad
      DATA deg2rad/0.0174532925d0/
      DATA ((DATDI(i,j),i=1,9),j=1,31)/  
     * -3., 0., 2., 0., 0.,-0.01, 0.0 , 0.0 , 0.0,   
     * -3., 2., 0., 0., 0.,-0.01, 0.0 , 0.0 , 0.0,   
     * -2., 0., 1.,-1., 0.,-0.02, 0.0 , 0.0 , 0.0,   
     * -2., 0., 1., 0., 0.,-0.08, 0.0 ,-0.01, 0.01,
     * -2., 2.,-1., 0., 0.,-0.02, 0.0 , 0.0 , 0.0,
     * -1., 0., 0.,-1., 0.,-0.10, 0.0 , 0.0 , 0.0,
     * -1., 0., 0., 0., 0.,-0.51, 0.0 ,-0.02, 0.03,
     * -1., 2., 0., 0., 0., 0.01, 0.0 , 0.0 , 0.0,
     *  0.,-2., 1., 0., 0., 0.01, 0.0 , 0.0 , 0.0,
     *  0., 0.,-1., 0., 0., 0.02, 0.0 , 0.0 , 0.0,
     *  0., 0., 1., 0., 0., 0.06, 0.0 , 0.0 , 0.0,
     *  0., 0., 1., 1., 0., 0.01, 0.0 , 0.0 , 0.0,
     *  0., 2.,-1., 0., 0., 0.01, 0.0 , 0.0 , 0.0,
     *  1.,-3., 0., 0., 1.,-0.06, 0.0 , 0.0 , 0.0,
     *  1.,-2., 0.,-1., 0., 0.01, 0.0 , 0.0 , 0.0,
     *  1.,-2., 0., 0., 0.,-1.23,-0.07, 0.06, 0.01,
     *  1.,-1., 0., 0.,-1., 0.02, 0.0 , 0.0 , 0.0,
     *  1.,-1., 0., 0., 1., 0.04, 0.0 , 0.0 , 0.0,
     *  1., 0., 0.,-1., 0.,-0.22, 0.01, 0.01, 0.0,
     *  1., 0., 0., 0., 0.,12.00,-0.80,-0.67,-0.03,
     *  1., 0., 0., 1., 0., 1.73,-0.12,-0.10, 0.0,
     *  1., 0., 0., 2., 0.,-0.04, 0.0 , 0.0 , 0.0, 
     *  1., 1., 0., 0.,-1.,-0.50,-0.01, 0.03, 0.0,
     *  1., 1., 0., 0., 1., 0.01, 0.0 , 0.0 , 0.0,
     *  0., 1., 0., 1.,-1.,-0.01, 0.0 , 0.0 , 0.0,
     *  1., 2.,-2., 0., 0.,-0.01, 0.0 , 0.0 , 0.0,
     *  1., 2., 0., 0., 0.,-0.11, 0.01, 0.01, 0.0,
     *  2.,-2., 1., 0., 0.,-0.01, 0.0 , 0.0 , 0.0,
     *  2., 0.,-1., 0., 0.,-0.02, 0.0 , 0.0 , 0.0,
     *  3., 0., 0., 0., 0., 0.0 , 0.0 , 0.0 , 0.0,
     *  3., 0., 0., 1., 0., 0.0 , 0.0 , 0.0 , 0.0/
      S=218.31664563D0+481267.88194D0*T-0.0014663889D0*T**2 
     1 +0.00000185139D0*T**3 
      TAU=fhr*15.D0+280.4606184D0+36000.7700536D0*T+0.00038793D0*T**2 
     1 -0.0000000258D0*T**3-S 
      PR=1.396971278*T+0.000308889*T**2+0.000000021*T**3 
     1 +0.000000007*T**4 
      S=S+PR 
      H=280.46645D0+36000.7697489D0*T+0.00030322222D0*T**2 
     1 +0.000000020*T**3-0.00000000654*T**4  
      P=83.35324312D0+4069.01363525D0*T-0.01032172222D0*T**2
     1 -0.0000124991D0*T**3+0.00000005263D0*T**4  
      ZNS=234.95544499D0 +1934.13626197D0*T-0.00207561111D0*T**2
     1 -0.00000213944D0*T**3+0.00000001650D0*T**4  
      PS=282.93734098D0+1.71945766667D0*T+0.00045688889D0*T**2 
     1 -0.00000001778D0*T**3-0.00000000334D0*T**4
C Reduce angles to between 0 and 360.
      s=  dmod(s,360.d0)
      tau=dmod(tau,360.d0)
      h=  dmod(h,360.d0)
      p=  dmod(p,360.d0)
      zns=dmod(zns,360.d0)
      ps=dmod(ps,360.d0)
c      WRITE(2,'(6f10.3)') tau,s,h,p,zns,ps

      RSTA=DSQRT(XSTA(1)**2+XSTA(2)**2+XSTA(3)**2)  
      SINPHI=XSTA(3)/RSTA  
      COSPHI=DSQRT(XSTA(1)**2+XSTA(2)**2)/RSTA  

      COSLA=XSTA(1)/COSPHI/RSTA
      SINLA=XSTA(2)/COSPHI/RSTA
      ZLA = DATAN2(XSTA(2),XSTA(1))
      DO 99 I=1,3  
      XCORSTA(I)=0.
99    continue
      DO 98 J=1,31
      THETAF=(TAU+DATDI(1,J)*S+DATDI(2,J)*H+DATDI(3,J)*P+
     1 DATDI(4,J)*ZNS+DATDI(5,J)*PS)*deg2rad
      DR=DATDI(6,J)*2.*SINPHI*COSPHI*SIN(THETAF+ZLA)+
     1 DATDI(7,J)*2.*SINPHI*COSPHI*COS(THETAF+ZLA)
      DN=DATDI(8,J)*(COSPHI**2-SINPHI**2)*SIN(THETAF+ZLA)+
     1 DATDI(9,J)*(COSPHI**2-SINPHI**2)*COS(THETAF+ZLA)
c      DE=DATDI(8,J)*SINPHI*COS(THETAF+ZLA)+
c     Modified 20 June 2007
      DE=DATDI(8,J)*SINPHI*COS(THETAF+ZLA)-
     1 DATDI(9,J)*SINPHI*SIN(THETAF+ZLA)
      XCORSTA(1)=XCORSTA(1)+DR*COSLA*COSPHI-DE*SINLA  
     1 -DN*SINPHI*COSLA  
      XCORSTA(2)=XCORSTA(2)+DR*SINLA*COSPHI+DE*COSLA  
     1 -DN*SINPHI*SINLA  
      XCORSTA(3)=XCORSTA(3)+DR*SINPHI+DN*COSPHI  
98    CONTINUE   

      DO 97 I=1,3
      XCORSTA(I)=XCORSTA(I)/1000.
97    CONTINUE  
      RETURN  
      END  
C  
C  *************************************************************  
C
      SUBROUTINE STEP2LON(XSTA,FHR,T,XCORSTA)  
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DOUBLE PRECISION deg2rad
      DOUBLE PRECISION XSTA(3),XCORSTA(3),DATDI(9,5)
      DATA deg2rad/0.0174532925d0/ 
      DATA ((DATDI(i,j),i=1,9),j=1,5)/  
     *   0, 0, 0, 1, 0,   0.47, 0.23, 0.16, 0.07,
     *   0, 2, 0, 0, 0,  -0.20,-0.12,-0.11,-0.05,
     *   1, 0,-1, 0, 0,  -0.11,-0.08,-0.09,-0.04,
     *   2, 0, 0, 0, 0,  -0.13,-0.11,-0.15,-0.07,
     *   2, 0, 0, 1, 0,  -0.05,-0.05,-0.06,-0.03/
C
      S=218.31664563D0+481267.88194D0*T-0.0014663889D0*T**2 
     1 +0.00000185139D0*T**3 
      PR=1.396971278*T+0.000308889*T**2+0.000000021*T**3 
     1 +0.000000007*T**4 
      S=S+PR 
      H=280.46645D0+36000.7697489D0*T+0.00030322222D0*T**2 
     1 +0.000000020*T**3-0.00000000654*T**4  
      P=83.35324312D0+4069.01363525D0*T-0.01032172222D0*T**2 
     1 -0.0000124991D0*T**3+0.00000005263D0*T**4  
      ZNS=234.95544499D0 +1934.13626197D0*T-0.00207561111D0*T**2
     1 -0.00000213944D0*T**3+0.00000001650D0*T**4  
      PS=282.93734098D0+1.71945766667D0*T+0.00045688889D0*T**2 
     1 -0.00000001778D0*T**3-0.00000000334D0*T**4  
      RSTA=DSQRT(XSTA(1)**2+XSTA(2)**2+XSTA(3)**2)  
      SINPHI=XSTA(3)/RSTA  
      COSPHI=DSQRT(XSTA(1)**2+XSTA(2)**2)/RSTA  
      COSLA=XSTA(1)/COSPHI/RSTA
      SINLA=XSTA(2)/COSPHI/RSTA
C reduce angles to between 0 and 360.
      s=  dmod(s,360.d0)
c      tau=dmod(tau,360.d0)
      h=  dmod(h,360.d0)
      p=  dmod(p,360.d0)
      zns=dmod(zns,360.d0)
      ps=dmod(ps,360.d0)

      dr_tot=0.
      dn_tot=0.
      DO 99 I=1,3  
      XCORSTA(I)=0.
99    continue
      DO 98 J=1,5
      THETAF=(DATDI(1,J)*S+DATDI(2,J)*H+DATDI(3,J)*P+
     1 DATDI(4,J)*ZNS+DATDI(5,J)*PS)*DEG2RAD
      DR=DATDI(6,J)*(3.*SINPHI**2-1.)/2.*COS(THETAF)+
     1     DATDI(8,J)*(3.*SINPHI**2-1.)/2.*SIN(THETAF)
      DN=DATDI(7,J)*(COSPHI*SINPHI*2.)*COS(THETAF)+
     1     DATDI(9,J)*(COSPHI*SINPHI*2.)*SIN(THETAF)
      DE=0. 
      dr_tot=dr_tot+dr
      dn_tot=dn_tot+dn
      XCORSTA(1)=XCORSTA(1)+DR*COSLA*COSPHI-DE*SINLA  
     1 -DN*SINPHI*COSLA  
      XCORSTA(2)=XCORSTA(2)+DR*SINLA*COSPHI+DE*COSLA  
     1 -DN*SINPHI*SINLA  
      XCORSTA(3)=XCORSTA(3)+DR*SINPHI+DN*COSPHI  
98    CONTINUE   

      DO 97 I=1,3
      XCORSTA(I)=XCORSTA(I)/1000.
97    CONTINUE  
      RETURN  
      END  
C**************************************************************************************************
C-------------------------------------------------------------------------
C 
      SUBROUTINE ST1IDIU(XSTA,XSUN,XMON,FAC2SUN,FAC2MON,XCORSTA)
C  
C THIS SUBROUTINE GIVES THE OUT-OF-PHASE CORRECTIONS INDUCED BY 
C MANTLE INELASTICITY IN THE DIURNAL BAND 
C  
C       INPUT : XSTA,XSUN,XMON,FAC2SUN,FAC2MON  
C      OUTPUT : XCORSTA  
C  

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DOUBLE PRECISION norm8
      DIMENSION XSTA(3),XSUN(3),XMON(3),XCORSTA(3)  
      DATA DHI/-0.0025/,DLI/-0.0007/  
      RSTA=NORM8(XSTA)
      SINPHI=XSTA(3)/RSTA  
      COSPHI=DSQRT(XSTA(1)**2+XSTA(2)**2)/RSTA
      COS2PHI=COSPHI**2-SINPHI**2
      SINLA=XSTA(2)/COSPHI/RSTA  
      COSLA=XSTA(1)/COSPHI/RSTA  
      RMON=NORM8(XMON)
      RSUN=NORM8(XSUN)
      DRSUN=-3.*DHI*SINPHI*COSPHI*FAC2SUN*XSUN(3)*(XSUN(1)*
     1            SINLA-XSUN(2)*COSLA)/RSUN**2
      DRMON=-3.*DHI*SINPHI*COSPHI*FAC2MON*XMON(3)*(XMON(1)*
     1            SINLA-XMON(2)*COSLA)/RMON**2
      DNSUN=-3.*DLI*COS2PHI*FAC2SUN*XSUN(3)*(XSUN(1)*SINLA-
     1            XSUN(2)*COSLA)/RSUN**2
      DNMON=-3.*DLI*COS2PHI*FAC2MON*XMON(3)*(XMON(1)*SINLA-
     1            XMON(2)*COSLA)/RMON**2
      DESUN=-3.*DLI*SINPHI*FAC2SUN*XSUN(3)*
     1 (XSUN(1)*COSLA+XSUN(2)*SINLA)/RSUN**2
      DEMON=-3.*DLI*SINPHI*FAC2MON*XMON(3)*
     1 (XMON(1)*COSLA+XMON(2)*SINLA)/RMON**2
      DR=DRSUN+DRMON 
      DN=DNSUN+DNMON  
      DE=DESUN+DEMON 
      XCORSTA(1)=DR*COSLA*COSPHI-DE*SINLA-DN*SINPHI*COSLA  
      XCORSTA(2)=DR*SINLA*COSPHI+DE*COSLA-DN*SINPHI*SINLA  
      XCORSTA(3)=DR*SINPHI+DN*COSPHI  
      RETURN  
      END  
C
C*********************************************************************
      DOUBLE PRECISION function norm8(a)
      DOUBLE PRECISION a(3)
      norm8=dSQRT(a(1)*a(1)+a(2)*a(2)+a(3)*a(3))
      return
      end
C

      SUBROUTINE iau_CAL2JD ( IY, IM, ID, DJM0, DJM, J )
*+
*  - - - - - - - - - - -
*   i a u _ C A L 2 J D
*  - - - - - - - - - - -
*
*  Gregorian Calendar to Julian Date.
*
*  This routine is part of the International Astronomical Union's
*  SOFA (Standards of Fundamental Astronomy) software collection.
*
*  Status:  support routine.
*
*  Given:
*     IY,IM,ID    i     year, month, day in Gregorian calendar (Note 1)
*
*  Returned:
*     DJM0        d     MJD zero-point: always 2400000.5
*     DJM         d     Modified Julian Date for 0 hrs
*     J           i     status:
*                           0 = OK
*                          -1 = bad year   (Note 3: JD not computed)
*                          -2 = bad month  (JD not computed)
*                          -3 = bad day    (JD computed)
*
*  Notes:
*
*  1) The algorithm used is valid from -4800 March 1, but this
*     implementation rejects dates before -4799 January 1.
*
*  2) The Julian Date is returned in two pieces, in the usual SOFA
*     manner, which is designed to preserve time resolution.  The
*     Julian Date is available as a single number by adding DJM0 and
*     DJM.
*
*  3) In early eras the conversion is from the "Proleptic Gregorian
*     Calendar";  no account is taken of the date(s) of adoption of
*     the Gregorian Calendar, nor is the AD/BC numbering convention
*     observed.
*
*  Reference:
*
*     Explanatory Supplement to the Astronomical Almanac,
*     P. Kenneth Seidelmann (ed), University Science Books (1992),
*     Section 12.92 (p604).
*
*  This revision:  2001 September 16
*
*  Copyright (C) 2007 IAU SOFA Review Board.  See notes at end.
*
*-----------------------------------------------------------------------

      IMPLICIT NONE

      INTEGER IY, IM, ID
      DOUBLE PRECISION DJM0, DJM
      INTEGER J, MY, IYPMY

*  Earliest year allowed (4800BC)
      INTEGER IYMIN
      PARAMETER ( IYMIN = -4799 )

*  Month lengths in days
      INTEGER MTAB(12)
      DATA MTAB / 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 /

* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

*  Preset status.
      J = 0

*  Validate year.
      IF ( IY.LT.IYMIN ) THEN
         J = -1
      ELSE

*     Validate month.
         IF ( IM.GE.1 .AND. IM.LE.12 ) THEN

*        Allow for leap year.
            IF ( MOD(IY,4) .EQ. 0 ) THEN
               MTAB(2) = 29
            ELSE
               MTAB(2) = 28
            END IF
            IF ( MOD(IY,100).EQ.0 .AND. MOD(IY,400).NE.0 ) MTAB(2) = 28

*        Validate day.
            IF ( ID.LT.1 .OR. ID.GT.MTAB(IM) ) J = -3

*        Result.
            MY = ( IM - 14 ) / 12
            IYPMY = IY + MY
            DJM0 = 2400000.5D0
            DJM = DBLE( ( 1461 * ( IYPMY + 4800 ) ) / 4
     :                + (  367 * ( IM-2 - 12*MY ) ) / 12
     :                - (    3 * ( ( IYPMY + 4900 ) / 100 ) ) / 4
     :                + ID - 2432076)

*        Bad month
         ELSE
            J = -2
         END IF
      END IF

*  Finished.

*+----------------------------------------------------------------------
*
*  Copyright (C) 2007
*  Standards Of Fundamental Astronomy Review Board
*  of the International Astronomical Union.
*
*  =====================
*  SOFA Software License
*  =====================
*
*  NOTICE TO USER:
*
*  BY USING THIS SOFTWARE YOU ACCEPT THE FOLLOWING TERMS AND CONDITIONS
*  WHICH APPLY TO ITS USE.
*
*  1. The Software is owned by the IAU SOFA Review Board ("the Board").
*
*  2. The Software is made available free of charge for use by:
*
*     a) private individuals for non-profit research; and
*
*     b) non-profit educational, academic and research institutions.
*
*  3. Commercial use of the Software is specifically excluded from the
*     terms and conditions of this license.  Commercial use of the
*     Software is subject to the prior written agreement of the Board on
*     terms to be agreed.
*
*  4. The provision of any version of the Software under the terms and
*     conditions specified herein does not imply that future versions
*     will also be made available under the same terms and conditions.
*
*  5. The user may modify the Software for his/her own purposes.  The
*     user may distribute the modified software provided that the Board
*     is informed and that a copy of the modified software is made
*     available to the Board on request.  All modifications made by the
*     user shall be clearly identified to show how the modified software
*     differs from the original Software, and the name(s) of the
*     affected routine(s) shall be changed.  The original SOFA Software
*     License text must be present.
*
*  6. In any published work produced by the user and which includes
*     results achieved by using the Software, the user shall acknowledge
*     that the Software was used in producing the information contained
*     in such publication.
*
*  7. The user may incorporate or embed the Software into other software
*     products which he/she may then give away free of charge but not
*     sell provided the user makes due acknowledgement of the use which
*     he/she has made of the Software in creating such software
*     products.  Any redistribution of the Software in this way shall be
*     made under the same terms and conditions under which the user
*     received it from the SOFA Center.
*
*  8. The user shall not cause the Software to be brought into
*     disrepute, either by misuse, or use for inappropriate tasks, or by
*     inappropriate modification.
*
*  9. The Software is provided to the user "as is" and the Board makes
*     no warranty as to its use or performance.   The Board does not and
*     cannot warrant the performance or results which the user may
*     obtain by using the Software.  The Board makes no warranties,
*     express or implied, as to non-infringement of third party rights,
*     merchantability, or fitness for any particular purpose.  In no
*     event will the Board be liable to the user for any consequential,
*     incidental, or special damages, including any lost profits or lost
*     savings, even if a Board representative has been advised of such
*     damages, or for any claim by any third party.
*
*  Correspondence concerning SOFA software should be addressed as
*  follows:
*
*     Internet email: sofa@rl.ac.uk
*     Postal address: IAU SOFA Center
*                     Rutherford Appleton Laboratory
*                     Chilton, Didcot, Oxon OX11 0QX
*                     United Kingdom
*
*
*-----------------------------------------------------------------------

      END
      SUBROUTINE ST1ISEM(XSTA,XSUN,XMON,FAC2SUN,FAC2MON,XCORSTA)
C  
C THIS SUBROUTINE GIVES THE OUT-OF-PHASE CORRECTIONS INDUCED BY 
C MANTLE INELASTICITY IN THE DIURNAL BAND 
C  
C       INPUT : XSTA,XSUN,XMON,FAC2SUN,FAC2MON  
C      OUTPUT : XCORSTA  
C  

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DOUBLE PRECISION norm8
      DIMENSION XSTA(3),XSUN(3),XMON(3),XCORSTA(3)  
      DATA DHI/-0.0022/,DLI/-0.0007/  
      RSTA=NORM8(XSTA)
      SINPHI=XSTA(3)/RSTA  
      COSPHI=DSQRT(XSTA(1)**2+XSTA(2)**2)/RSTA
      SINLA=XSTA(2)/COSPHI/RSTA  
      COSLA=XSTA(1)/COSPHI/RSTA  
      COSTWOLA=COSLA**2-SINLA**2  
      SINTWOLA=2.*COSLA*SINLA  
      RMON=NORM8(XMON)
      RSUN=NORM8(XSUN)
      DRSUN=-3./4.*DHI*COSPHI**2*FAC2SUN*((XSUN(1)**2-XSUN(2)**2)*  
     1 SINTWOLA-2.*XSUN(1)*XSUN(2)*COSTWOLA)/RSUN**2
      DRMON=-3./4.*DHI*COSPHI**2*FAC2MON*((XMON(1)**2-XMON(2)**2)*  
     1 SINTWOLA-2.*XMON(1)*XMON(2)*COSTWOLA)/RMON**2
      DNSUN=3./2.*DLI*SINPHI*COSPHI*FAC2SUN*((XSUN(1)**2-XSUN(2)**2)*  
     1 SINTWOLA-2.*XSUN(1)*XSUN(2)*COSTWOLA)/RSUN**2
      DNMON=3./2.*DLI*SINPHI*COSPHI*FAC2MON*((XMON(1)**2-XMON(2)**2)*  
     1 SINTWOLA-2.*XMON(1)*XMON(2)*COSTWOLA)/RMON**2
      DESUN=-3./2.*DLI*COSPHI*FAC2SUN*((XSUN(1)**2-XSUN(2)**2)*  
     1 COSTWOLA+2.*XSUN(1)*XSUN(2)*SINTWOLA)/RSUN**2
      DEMON=-3./2.*DLI*COSPHI*FAC2MON*((XMON(1)**2-XMON(2)**2)*  
     1 COSTWOLA+2.*XMON(1)*XMON(2)*SINTWOLA)/RMON**2
      DR=DRSUN+DRMON 
      DN=DNSUN+DNMON  
      DE=DESUN+DEMON 
      XCORSTA(1)=DR*COSLA*COSPHI-DE*SINLA-DN*SINPHI*COSLA  
      XCORSTA(2)=DR*SINLA*COSPHI+DE*COSLA-DN*SINPHI*SINLA  
      XCORSTA(3)=DR*SINPHI+DN*COSPHI  
      RETURN  
      END  
C
C*********************************************************************
      subroutine zero_vec8(v)
      DOUBLE PRECISION v(3)
      v(1)=0.
      v(2)=0.
      v(3)=0.
      return
      end
C
C*********************************************************************
      SUBROUTINE iau_DAT ( IY, IM, ID, FD, DELTAT, J )
*+
*  - - - - - - - -
*   i a u _ D A T
*  - - - - - - - -
*
*  For a given UTC date, calculate delta(AT) = TAI-UTC.
*
*     :------------------------------------------:
*     :                                          :
*     :                 IMPORTANT                :
*     :                                          :
*     :  A new version of this routine must be   :
*     :  produced whenever a new leap second is  :
*     :  announced.  There are five items to     :
*     :  change on each such occasion:           :
*     :                                          :
*     :  1) The parameter NDAT must be           :
*     :     increased by 1.                      :
*     :                                          :
*     :  2) A new line must be added to the set  :
*     :     of DATA statements that initialize   :
*     :     the arrays IDATE and DATS.           :
*     :                                          :
*     :  3) The parameter IYV must be set to     :
*     :     the current year.                    :
*     :                                          :
*     :  4) The "Latest leap second" comment     :
*     :     below must be set to the new leap    :
*     :     second date.                         :
*     :                                          :
*     :  5) The "This revision" comment, later,  :
*     :     must be set to the current date.     :
*     :                                          :
*     :  Change (3) must also be carried out     :
*     :  whenever the routine is re-issued,      :
*     :  even if no leap seconds have been       :
*     :  added.                                  :
*     :                                          :
*     :  Latest leap second:  2006 January 1     :
*     :                                          :
*     :__________________________________________:
*
*  This routine is part of the International Astronomical Union's
*  SOFA (Standards of Fundamental Astronomy) software collection.
*
*  Status:  support routine.
*
*  Given:
*     IY       i     UTC:  year (Notes 1 and 2)
*     IMO      i           month (Note 2)
*     ID       i           day (Notes 2 and 3)
*     FD       d           fraction of day (Note 4)
*
*  Returned:
*     DELTAT   d     TAI minus UTC, seconds
*     J        i     status (Note 5):
*                       1 = dubious year (Note 1)
*                       0 = OK
*                      -1 = bad year
*                      -2 = bad month
*                      -3 = bad day (Note 3)
*                      -4 = bad fraction (Note 4)
*
*  Notes:
*
*  1) UTC began at 1960 January 1.0 (JD 2436934.5) and it is improper
*     to call the routine with an earlier epoch.  If this is attempted,
*     zero is returned together with a warning status.
*
*     Because leap seconds cannot, in principle, be predicted in
*     advance, a reliable check for dates beyond the valid range is
*     impossible.  To guard against gross errors, a year five or more
*     after the release year of the present routine (see parameter IYV)
*     is considered dubious.  In this case a warning status is returned
*     but the result is computed in the normal way.
*
*     For both too-early and too-late years, the warning status is J=+1.
*     This is distinct from the error status J=-1, which signifies a
*     year so early that JD could not be computed.
*
*  2) If the specified date is for a day which ends with a leap second,
*     the UTC-TAI value returned is for the period leading up to the
*     leap second.  If the date is for a day which begins as a leap
*     second ends, the UTC-TAI returned is for the period following the
*     leap second.
*
*  3) The day number must be in the normal calendar range, for example
*     1 through 30 for April.  The "almanac" convention of allowing
*     such dates as January 0 and December 32 is not supported in this
*     routine, in order to avoid confusion near leap seconds.
*
*  4) The fraction of day is used only for dates before the introduction
*     of leap seconds, the first of which occurred at the end of 1971.
*     It is tested for validity (zero to less than 1 is the valid range)
*     even if not used;  if invalid, zero is used and status J=-4 is
*     returned.  For many applications, setting FD to zero is
*     acceptable;  the resulting error is always less than 3 ms (and
*     occurs only pre-1972).
*
*  5) The status value returned in the case where there are multiple
*     errors refers to the first error detected.  For example, if the
*     month and day are 13 and 32 respectively, JSTAT=-2 (bad month)
*     will be returned.
*
*  6) In cases where a valid result is not available, zero is returned.
*
*  References:
*
*  1) For epochs from 1961 January 1 onwards, the expressions from the
*     file ftp://maia.usno.navy.mil/ser7/tai-utc.dat are used.
*
*  2) The 5ms timestep at 1961 January 1 is taken from 2.58.1 (p87) of
*     the 1992 Explanatory Supplement.
*
*  Called:
*     iau_CAL2JD   Gregorian calendar to Julian Day number
*
*  This revision:  2006 November 13
*
*  Copyright (C) 2007 IAU SOFA Review Board.  See notes at end.
*
*-----------------------------------------------------------------------

      IMPLICIT NONE

      INTEGER IY, IM, ID
      DOUBLE PRECISION FD, DELTAT
      INTEGER J

*  Release year for this version of iau_DAT.
      INTEGER IYV
      PARAMETER ( IYV = 2005 )

*  Number of Delta(AT) changes (increase by 1 for each new leap second).
      INTEGER NDAT
      PARAMETER ( NDAT = 38 )

*  Number of Delta(AT) expressions before leap seconds were introduced.
      INTEGER NERA1
      PARAMETER ( NERA1 = 14 )

*  Dates (year, month) on which new Delta(AT) came into force.
      INTEGER IDATE(2,NDAT)

*  New Delta(AT) which came into force on the given dates.
      DOUBLE PRECISION DATS(NDAT)

*  Reference dates (MJD) and drift rates (s/day), pre leap seconds.
      DOUBLE PRECISION DRIFT(2,NERA1)

*  Miscellaneous local variables.
      LOGICAL MORE
      INTEGER JS, I, M, IS
      DOUBLE PRECISION DAT, DJM0, DJM

*  Dates and Delta(AT)s.
      DATA (IDATE(I, 1),I=1,2),DATS(1)  / 1960,  1,  1.4178180D0 /
      DATA (IDATE(I, 2),I=1,2),DATS(2)  / 1961,  1,  1.4228180D0 /
      DATA (IDATE(I, 3),I=1,2),DATS(3)  / 1961,  8,  1.3728180D0 /
      DATA (IDATE(I, 4),I=1,2),DATS(4)  / 1962,  1,  1.8458580D0 /
      DATA (IDATE(I, 5),I=1,2),DATS(5)  / 1963, 11,  1.9458580D0 /
      DATA (IDATE(I, 6),I=1,2),DATS(6)  / 1964,  1,  3.2401300D0 /
      DATA (IDATE(I, 7),I=1,2),DATS(7)  / 1964,  4,  3.3401300D0 /
      DATA (IDATE(I, 8),I=1,2),DATS(8)  / 1964,  9,  3.4401300D0 /
      DATA (IDATE(I, 9),I=1,2),DATS(9)  / 1965,  1,  3.5401300D0 /
      DATA (IDATE(I,10),I=1,2),DATS(10) / 1965,  3,  3.6401300D0 /
      DATA (IDATE(I,11),I=1,2),DATS(11) / 1965,  7,  3.7401300D0 /
      DATA (IDATE(I,12),I=1,2),DATS(12) / 1965,  9,  3.8401300D0 /
      DATA (IDATE(I,13),I=1,2),DATS(13) / 1966,  1,  4.3131700D0 /
      DATA (IDATE(I,14),I=1,2),DATS(14) / 1968,  2,  4.2131700D0 /
      DATA (IDATE(I,15),I=1,2),DATS(15) / 1972,  1, 10D0 /
      DATA (IDATE(I,16),I=1,2),DATS(16) / 1972,  7, 11D0 /
      DATA (IDATE(I,17),I=1,2),DATS(17) / 1973,  1, 12D0 /
      DATA (IDATE(I,18),I=1,2),DATS(18) / 1974,  1, 13D0 /
      DATA (IDATE(I,19),I=1,2),DATS(19) / 1975,  1, 14D0 /
      DATA (IDATE(I,20),I=1,2),DATS(20) / 1976,  1, 15D0 /
      DATA (IDATE(I,21),I=1,2),DATS(21) / 1977,  1, 16D0 /
      DATA (IDATE(I,22),I=1,2),DATS(22) / 1978,  1, 17D0 /
      DATA (IDATE(I,23),I=1,2),DATS(23) / 1979,  1, 18D0 /
      DATA (IDATE(I,24),I=1,2),DATS(24) / 1980,  1, 19D0 /
      DATA (IDATE(I,25),I=1,2),DATS(25) / 1981,  7, 20D0 /
      DATA (IDATE(I,26),I=1,2),DATS(26) / 1982,  7, 21D0 /
      DATA (IDATE(I,27),I=1,2),DATS(27) / 1983,  7, 22D0 /
      DATA (IDATE(I,28),I=1,2),DATS(28) / 1985,  7, 23D0 /
      DATA (IDATE(I,29),I=1,2),DATS(29) / 1988,  1, 24D0 /
      DATA (IDATE(I,30),I=1,2),DATS(30) / 1990,  1, 25D0 /
      DATA (IDATE(I,31),I=1,2),DATS(31) / 1991,  1, 26D0 /
      DATA (IDATE(I,32),I=1,2),DATS(32) / 1992,  7, 27D0 /
      DATA (IDATE(I,33),I=1,2),DATS(33) / 1993,  7, 28D0 /
      DATA (IDATE(I,34),I=1,2),DATS(34) / 1994,  7, 29D0 /
      DATA (IDATE(I,35),I=1,2),DATS(35) / 1996,  1, 30D0 /
      DATA (IDATE(I,36),I=1,2),DATS(36) / 1997,  7, 31D0 /
      DATA (IDATE(I,37),I=1,2),DATS(37) / 1999,  1, 32D0 /
      DATA (IDATE(I,38),I=1,2),DATS(38) / 2006,  1, 33D0 /

*  Reference dates and drift rates.
      DATA (DRIFT(I, 1),I=1,2) / 37300D0, 0.001296D0 /
      DATA (DRIFT(I, 2),I=1,2) / 37300D0, 0.001296D0 /
      DATA (DRIFT(I, 3),I=1,2) / 37300D0, 0.001296D0 /
      DATA (DRIFT(I, 4),I=1,2) / 37665D0, 0.0011232D0 /
      DATA (DRIFT(I, 5),I=1,2) / 37665D0, 0.0011232D0 /
      DATA (DRIFT(I, 6),I=1,2) / 38761D0, 0.001296D0 /
      DATA (DRIFT(I, 7),I=1,2) / 38761D0, 0.001296D0 /
      DATA (DRIFT(I, 8),I=1,2) / 38761D0, 0.001296D0 /
      DATA (DRIFT(I, 9),I=1,2) / 38761D0, 0.001296D0 /
      DATA (DRIFT(I,10),I=1,2) / 38761D0, 0.001296D0 /
      DATA (DRIFT(I,11),I=1,2) / 38761D0, 0.001296D0 /
      DATA (DRIFT(I,12),I=1,2) / 38761D0, 0.001296D0 /
      DATA (DRIFT(I,13),I=1,2) / 39126D0, 0.002592D0 /
      DATA (DRIFT(I,14),I=1,2) / 39126D0, 0.002592D0 /

* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

*  Initialize the result to zero and the status to OK.
      DAT = 0D0
      JS = 0

*  If invalid fraction of a day, set error status and give up.
      IF ( FD.LT.0D0 .OR. FD.GE.1D0 ) THEN
         JS = -4
         GO TO 9000
      END IF

*  Convert the date into an MJD.
      CALL iau_CAL2JD ( IY, IM, ID, DJM0, DJM, JS )

*  If invalid year, month, or day, give up.
      IF ( JS .LT. 0 ) GO TO 9000

*  If pre-UTC year, set warning status and give up.
      IF ( IY .LT. IDATE(1,1) ) THEN
         JS = 1
         GO TO 9000
      END IF

*  If suspiciously late year, set warning status but proceed.
      IF ( IY .GT. IYV+5 ) JS = 1

*  Combine year and month.
      M = 12*IY+IM

*  Prepare to search the tables.
      MORE = .TRUE.

*  Find the most recent table entry.
      DO 1 I=NDAT,1,-1
         IF ( MORE ) THEN
            IS = I
            MORE = M .LT. ( 12*IDATE(1,I) + IDATE(2,I) )
         END IF
 1    CONTINUE

*  Get the Delta(AT).
      DAT = DATS(IS)

*  If pre-1972, adjust for drift.
      IF ( IS .LE. NERA1 ) DAT = DAT +
     :                          ( DJM + FD - DRIFT(1,IS) ) * DRIFT(2,IS)

*  Return the Delta(AT) value and the status.
 9000 CONTINUE
      DELTAT = DAT
      J = JS

*  Finished.

*+----------------------------------------------------------------------
*
*  Copyright (C) 2007
*  Standards Of Fundamental Astronomy Review Board
*  of the International Astronomical Union.
*
*  =====================
*  SOFA Software License
*  =====================
*
*  NOTICE TO USER:
*
*  BY USING THIS SOFTWARE YOU ACCEPT THE FOLLOWING TERMS AND CONDITIONS
*  WHICH APPLY TO ITS USE.
*
*  1. The Software is owned by the IAU SOFA Review Board ("the Board").
*
*  2. The Software is made available free of charge for use by:
*
*     a) private individuals for non-profit research; and
*
*     b) non-profit educational, academic and research institutions.
*
*  3. Commercial use of the Software is specifically excluded from the
*     terms and conditions of this license.  Commercial use of the
*     Software is subject to the prior written agreement of the Board on
*     terms to be agreed.
*
*  4. The provision of any version of the Software under the terms and
*     conditions specified herein does not imply that future versions
*     will also be made available under the same terms and conditions.
*
*  5. The user may modify the Software for his/her own purposes.  The
*     user may distribute the modified software provided that the Board
*     is informed and that a copy of the modified software is made
*     available to the Board on request.  All modifications made by the
*     user shall be clearly identified to show how the modified software
*     differs from the original Software, and the name(s) of the
*     affected routine(s) shall be changed.  The original SOFA Software
*     License text must be present.
*
*  6. In any published work produced by the user and which includes
*     results achieved by using the Software, the user shall acknowledge
*     that the Software was used in producing the information contained
*     in such publication.
*
*  7. The user may incorporate or embed the Software into other software
*     products which he/she may then give away free of charge but not
*     sell provided the user makes due acknowledgement of the use which
*     he/she has made of the Software in creating such software
*     products.  Any redistribution of the Software in this way shall be
*     made under the same terms and conditions under which the user
*     received it from the SOFA Center.
*
*  8. The user shall not cause the Software to be brought into
*     disrepute, either by misuse, or use for inappropriate tasks, or by
*     inappropriate modification.
*
*  9. The Software is provided to the user "as is" and the Board makes
*     no warranty as to its use or performance.   The Board does not and
*     cannot warrant the performance or results which the user may
*     obtain by using the Software.  The Board makes no warranties,
*     express or implied, as to non-infringement of third party rights,
*     merchantability, or fitness for any particular purpose.  In no
*     event will the Board be liable to the user for any consequential,
*     incidental, or special damages, including any lost profits or lost
*     savings, even if a Board representative has been advised of such
*     damages, or for any claim by any third party.
*
*  Correspondence concerning SOFA software should be addressed as
*  follows:
*
*     Internet email: sofa@rl.ac.uk
*     Postal address: IAU SOFA Center
*                     Rutherford Appleton Laboratory
*                     Chilton, Didcot, Oxon OX11 0QX
*                     United Kingdom
*
*
*-----------------------------------------------------------------------

      END
C*********************************************************************
C      SUBROUTINE ARG(IYEAR,DAY,ANGLE)
C
C  COMPUTES THE ANGULAR ARGUMENT WHICH DEPENDS ON TIME FOR 11
C  TIDAL ARGUMENT CALCULATIONS
C
C
C    ORDER OF THE 11 ANGULAR QUANTITIES IN VECTOR ANGLE
C
C  01-M2
C  02-S2
C  03-N2
C  04-K2
C  05-K1
C  06-O1
C  07-P1
C  08-Q1
C  09-Mf
C  10-Mm
C  11-Ssa
C
C  TAKEN FROM 'TABLE 1 CONSTANTS OF MAJOR TIDAL MODES'
C       WHICH DR. SCHWIDERSKI SENDS ALONG WITH HIS TAPE OF TIDAL
C       AMPLITUDES AND PHASES
C
C
C  INPUT--
C
C     IYEAR - EX. 79 FOR 1979
C     DAY - DAY OF YEAR GREENWICH TIME
C                      EXAMPLE 32.5 FOR FEB 1   12 NOON
C                              1.25 FOR JAN 1    6 AM
C
C  OUTPUT--
C
C     ANGLE - ANGULAR ARGUMENT FOR SCHWIDERSKI COMPUTATION
C
C***********************************************************
C
C
C              C A U T I O N
C
C  OCEAN LOADING PHASES COMPUTED FROM SCHWIDERSKI'S MODELS
C  REFER TO THE PHASE OF THE ASSOCIATED SOLID EARTH TIDE
C  GENERATING POTENTIAL AT THE ZERO MERIDIAN ACCORDING TO
C
C      OL_DR = OL_AMP ' COS (SE_PHASE" - OL_PHASE)
C
C  WHERE OL = OCEAN LOADING TIDE,
C        SE = SOLID EARTH TIDE GENERATING POTENTIAL.
C
C  IF THE HARMONIC TIDE DEVELOPMENT OF CARTWRIGHT, ET AL.
C  ( = CTE) (1971, 1973) IS USED, MAKE SURE THAT SE_PHASE"
C  TAKES INTO ACCOUNT
C
C  (1) THE SIGN OF SE_AMP IN THE TABLES OF CARTWRIGHT ET AL.
C
C  (2) THAT CTE'S SE_PHASE REFERS TO A SINE RATHER THAN A
C      COSINE FUNCTION IF (N+M) = (DEGREE + ORDER) OF THE
C      TIDE SPHERICAL HARMONIC IS ODD.
C
C  I.E. SE_PHASE" = TAU(T) ' N1 + S(T) ' N2 + H(T) ' N3
C                 + P(T) ' N4 + N'(T) ' N5 + PS(T) ' N6
C                 + PI   IF CTE'S AMPLITUDE COEFFICIENT < 0
C                 + PI/2 IF (DEGREE + N1) IS ODD
C
C  WHERE TAU ... PS = ASTRONOMICAL ARGUMENTS,
C         N1 ... N6 = CTE'S ARGUMENT NUMBERS.
C
C  MOST TIDE GENERATING SOFTWARE COMPUTE SE_PHASE" (FOR
C  USE WITH COSINES).
C
C  THIS SUBROUTINE IS VALID ONLY AFTER 1973.
C
C*******************************************************************
      SUBROUTINE ARG(IYEAR,DAY,ANGLE)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      REAL ANGFAC(4,11)
      DIMENSION ANGLE(11),SPEED(11)
C
C  SPEED OF ALL TERMS IN RADIANS PER SEC
C
      EQUIVALENCE (SPEED(1),SIGM2),(SPEED(2),SIGS2),(SPEED(3),SIGN2)
      EQUIVALENCE (SPEED(4),SIGK2),(SPEED(5),SIGK1),(SPEED(6),SIGO1)
      EQUIVALENCE (SPEED(7),SIGP1),(SPEED(8),SIGQ1),(SPEED(9),SIGMF)
      EQUIVALENCE (SPEED(10),SIGMM),(SPEED(11),SIGSSA)
      DATA SIGM2/1.40519D-4/
      DATA SIGS2/1.45444D-4/
      DATA SIGN2/1.37880D-4/
      DATA SIGK2/1.45842D-4/
      DATA SIGK1/.72921D-4/
      DATA SIGO1/.67598D-4/
      DATA SIGP1/.72523D-4/
      DATA SIGQ1/.64959D-4/
      DATA SIGMF/.053234D-4/
      DATA SIGMM/.026392D-4/
      DATA SIGSSA/.003982D-4/
      DATA ANGFAC/2.E0,-2.E0,0.E0,0.E0,4*0.E0,
     .            2.E0,-3.E0,1.E0,0.E0,2.E0,3*0.E0,
     .            1.E0,2*0.E0,.25E0,1.E0,-2.E0,0.E0,-.25E0,
     .            -1.E0,2*0.E0,-.25E0,1.E0,-3.E0,1.E0,-.25E0,
     .            0.E0,2.E0,2*0.E0,0.E0,1.E0,-1.E0,0.E0,
     .            2.E0,3*0.E0/
      DATA TWOPI/6.28318530718D0/
      DATA DTR/.174532925199D-1/
C
C  DAY OF YEAR
C
      ID=DAY
C
C  FRACTIONAL PART OF DAY IN SECONDS
C
      FDAY=(DAY-ID)*86400.D0
      ICAPD=ID+365*(IYEAR-75)+((IYEAR-73)/4)
      CAPT=(27392.500528D0+1.000000035D0*ICAPD)/36525.D0

C
C MEAN LONGITUDE OF SUN AT BEGINNING OF DAY
C
      H0=(279.69668D0+(36000.768930485D0+3.03D-4*CAPT)*CAPT)*DTR
C
C MEAN LONGITUDE OF MOON AT BEGINNING OF DAY
C
      S0=(((1.9D-6*CAPT-.001133D0)*CAPT+481267.88314137D0)*CAPT
     .  +270.434358D0)*DTR
C
C MEAN LONGITUDE OF LUNAR PERIGEE AT BEGINNING OF DAY
C
      P0=(((-1.2D-5*CAPT-.010325D0)*CAPT+4069.0340329577D0)*CAPT
     .  +334.329653D0)*DTR
      DO 500 K=1,11
      ANGLE(K)=SPEED(K)*FDAY+ANGFAC(1,K)*H0+ANGFAC(2,K)*S0
     .  +ANGFAC(3,K)*P0+ANGFAC(4,K)*TWOPI
      ANGLE(K)=DMOD(ANGLE(K),TWOPI)
      IF(ANGLE(K).LT.0.D0)ANGLE(K)=ANGLE(K)+TWOPI
  500 CONTINUE
      RETURN
      END

